// A. Sints
#include "stdafx.h"

#include "Stuka.h"
#include "AdjustmentsDlg.h"


// math
#include <math.h>


// CAdjustmentsDlg dialog
IMPLEMENT_DYNAMIC( CAdjustmentsDlg, CDialog )



// dtor
CAdjustmentsDlg::~CAdjustmentsDlg() {

	// delete objects
	delete m_pFFMPEGCapture;
	delete m_pInput;

	// delete bitmap
	DeleteObject( m_hBmp );
}


// data exchange
void CAdjustmentsDlg::DoDataExchange( CDataExchange * _pDX ) {
	CDialog::DoDataExchange( _pDX );
	DDX_Control( _pDX, IDC_COLOR_CR, m_color_cr );
	DDX_Control( _pDX, IDC_COLOR_MG, m_color_mg );
	DDX_Control( _pDX, IDC_COLOR_YB, m_color_yb );
	DDX_Control( _pDX, IDC_BRIGHTNESS, m_brightness );
	DDX_Control( _pDX, IDC_CONTRAST, m_contrast );
	DDX_Control( _pDX, IDC_SATURATION, m_saturation );
	DDX_Control( _pDX, IDC_DUMMY, m_dummy );
	DDX_Control( _pDX, IDC_DNR, m_dnr );
	DDX_Control( _pDX, IDC_COMBO1, m_vflip );
}


// message map
BEGIN_MESSAGE_MAP( CAdjustmentsDlg, CDialog )
	ON_WM_PAINT()
	ON_BN_CLICKED( IDOK, &CAdjustmentsDlg::OnBnClickedOk )
	ON_WM_HSCROLL()
	ON_WM_TIMER()
	ON_BN_CLICKED( IDC_LOAD, &CAdjustmentsDlg::OnBnClickedLoad )
	ON_CBN_SELCHANGE(IDC_COMBO1, &CAdjustmentsDlg::OnCbnSelchangeCombo1)
END_MESSAGE_MAP()



// on init dlg
BOOL CAdjustmentsDlg::OnInitDialog() {

	// call parent
	CDialog::OnInitDialog();

	// get dummy rect
	m_dummy.GetWindowRect( &m_rctDraw );
	ScreenToClient( m_rctDraw );
	//m_rctDraw.right -= 10;

	// create bitmap info header	
	BITMAPINFO bmpInfo;
	ZeroMemory( &bmpInfo, sizeof( BITMAPINFO ) );
	bmpInfo.bmiHeader.biSize = sizeof( BITMAPINFOHEADER );
	bmpInfo.bmiHeader.biBitCount = 32;
	bmpInfo.bmiHeader.biWidth = m_rctDraw.Width();
	bmpInfo.bmiHeader.biHeight = m_rctDraw.Height();
	bmpInfo.bmiHeader.biPlanes = 1;
	bmpInfo.bmiHeader.biSizeImage = m_rctDraw.Height() * m_rctDraw.Width() * 4;

	// create DIBSection
	m_hBmp = CreateDIBSection( NULL, &bmpInfo, DIB_RGB_COLORS, ( void ** )&( m_lpBits ), NULL, 0 );

	// create bmp device context
	m_bmpDC.CreateCompatibleDC( GetDC() );
	m_bmpDC.SelectObject( m_hBmp );

	// init sliders
	CSliderCtrl * sliders[ 7 ] = { &m_color_cr, &m_color_mg, &m_color_yb, &m_brightness, &m_contrast, &m_saturation, &m_dnr };
	for( int i = 0; i < 7; i ++ ) {
		sliders[ i ]->SetLineSize( 100 );
		sliders[ i ]->SetPageSize( 50 );
		sliders[ i ]->SetTicFreq( 50 );
		sliders[ i ]->SetPos( 50 );
	}
	m_dnr.SetPos( 0 );

	// flip
	m_vflip.InsertString( 0, L"auto" );
	m_vflip.InsertString( 1, L"true" );
	m_vflip.InsertString( 2, L"false" );
	m_vflip.SetCurSel( 0 );

	// refresh timer
	m_dwPosition = 0;
	SetTimer( 1, 40, NULL );
	
	// no focus
	return TRUE;
}


// on timer
void CAdjustmentsDlg::OnTimer( UINT_PTR ) {

	// next frame
	m_dwPosition++;
	if( m_dwPosition == 100 )
		m_dwPosition = 0;

	// refresh
	InvalidateRect( m_rctDraw, FALSE );
}



// on paint
void CAdjustmentsDlg::OnPaint() {

	// iconic
	if( IsIconic() )
		return;

	// current dc
	CPaintDC dc( this );
	
	// nothing to do
	if( m_pFFMPEGCapture == NULL ) {
		CBrush brush( RGB( 127, 127, 127 ) );
		dc.FillRect( m_rctDraw, &brush );
		CDialog::OnPaint();
		return;
	}
	
	// get bitmap from FFMPEG
	HBITMAP hSrcBmp = NULL;
	while( hSrcBmp == NULL ) {
		hSrcBmp = m_pFFMPEGCapture->GetBitmap( m_dwPosition );
		if( m_pFFMPEGCapture->CaptureError() ) {
			delete m_pFFMPEGCapture;
			m_pFFMPEGCapture = NULL;
			delete m_pInput;
			m_pInput = NULL;
			MessageBox( L"Failed to start capture on video.", L"Adjustments", IDOK | MB_ICONERROR );
			return;
		}
	}

	// create bitmap info header
	BITMAP bmpSrc;
	GetObject( hSrcBmp, sizeof( BITMAP ), &bmpSrc );
	unsigned char * pu8Src = ( unsigned char * ) bmpSrc.bmBits;

	// adjustments values
	float fCR( float( m_color_cr.GetPos() - 50 ) / 50 );
	float fMG( float( m_color_mg.GetPos() - 50 ) / 50 );
	float fYB( float( m_color_yb.GetPos() - 50 ) / 50 );
	float fBri( float( m_brightness.GetPos() - 50 ) / 50 );
	float fCon( float( m_contrast.GetPos() - 50 ) / 25 );
	float fSat( float( m_saturation.GetPos() ) / 50 );

	// color balance
	unsigned char colorBalTable[ 3 ][ 256 ];
	double dR, dG, dB;
	double dRatio;
	for( int i = 0; i < 256; i++ ) {
		dRatio = 127 * sin( i * 0.012319971190548208778284876012861 ); // PI / 255
		dR = i + ( dRatio * fCR );
		colorBalTable[ 0 ][ i ] = dR > 255 ? 255 : ( dR < 0 ? 0 : unsigned char( dR ) );
		dG = i + ( dRatio * fMG );
		colorBalTable[ 1 ][ i ] = dG > 255 ? 255 : ( dG < 0 ? 0 : unsigned char( dG ) );
		dB = i + ( dRatio * fYB );
		colorBalTable[ 2 ][ i ] = dB > 255 ? 255 : ( dB < 0 ? 0 : unsigned char( dB ) );
	}

	// brightness
	unsigned char brightnessTable[ 256 ];
	double dBrightness;
	for( int i = 0; i < 256; i++ ) {
		dBrightness = i + ( ( 255 - i ) * fBri );
		brightnessTable[ i ] = dBrightness > 255 ? 255 : ( dBrightness < 0 ? 0 : unsigned char( dBrightness ) );
	}

	// contrast
	double contrastAdj[ 256 ];
	for( int i = 128; i < 256; i++ ) {
		contrastAdj[ i ] = ( 128 + ( sin( ( i - 128 ) * 0.012368475014133044245915918831809 ) * 127 ) ) - i; // ( PI / 2 ) / 127
		contrastAdj[ 255 - i ] = -contrastAdj[ i ];
	}

	unsigned char contrastTable[ 256 ];
	double dContrast;
	for( int i = 0; i < 256; i++ ) {
		dContrast = i + ( contrastAdj[ i ] * fCon );
		contrastTable[ i ] = i < 128 ? ( dContrast < 0 ? 0 : ( dContrast > 127 ? 127 : unsigned char( dContrast ) ) ) : ( dContrast < 127 ? 127 : ( dContrast > 255 ? 255 : unsigned char( dContrast ) ) );
	}

	// saturation
	unsigned char saturationTable[ 256 ][ 256 ];
	double dSaturation;
	for( int i = 0; i < 256; i++ ) {
		for( int j = 0; j < 256; j++ ) {
			dSaturation = j + int( double( i - j ) * fSat );
			saturationTable[ i ][ j ] = dSaturation > 255 ? 255 : ( dSaturation < 0 ? 0 : unsigned char( dSaturation ) );
		}
	}

	// copy bits to bitmap
	DWORD dwRGBWidth = DWORD( m_rctDraw.Width() ) * 3;
	DWORD dwRGBWidthDst = DWORD( m_rctDraw.Width() ) * 4;
	DWORD dwLine, dwLineDst, dwPos, dwPosDst;
	int iRcb, iGcb, iBcb, iRGBc;
	for( DWORD y = 0; y < DWORD( m_rctDraw.Height() ); y ++ ) {
		dwLine = dwRGBWidth * y;
		dwLineDst = dwRGBWidthDst * y;
		for( DWORD x = 0; x < DWORD( m_rctDraw.Width() ); x ++ ) {
			dwPos = dwLine + ( x * 3 );
			dwPosDst = dwLineDst + ( x * 4 );

			// apply color balance, brightness and contrast
			iRcb = contrastTable[ brightnessTable[ colorBalTable[ 2 ][ pu8Src[ dwPos + 0 ] ] ] ];
			iGcb = contrastTable[ brightnessTable[ colorBalTable[ 1 ][ pu8Src[ dwPos + 1 ] ] ] ];
			iBcb = contrastTable[ brightnessTable[ colorBalTable[ 0 ][ pu8Src[ dwPos + 2 ] ] ] ];
			iRGBc = ( iRcb + iGcb + iBcb ) / 3;
			// apply saturation and set result
			m_lpBits[ dwPosDst ]		= saturationTable[ iRcb ][ iRGBc ];
			m_lpBits[ dwPosDst + 1 ]	= saturationTable[ iGcb ][ iRGBc ];
			m_lpBits[ dwPosDst + 2 ]	= saturationTable[ iBcb ][ iRGBc ];
			m_lpBits[ dwPosDst + 3 ]	= 255;
		}
	}

	// draw bitmap to Dlg
	dc.BitBlt( m_rctDraw.left, m_rctDraw.top, m_rctDraw.Width(), m_rctDraw.Height(), &m_bmpDC, 0, 0, SRCCOPY );

	// parent call
	CDialog::OnPaint();
}

// copy values to clipboard
void CAdjustmentsDlg::CopyToClipBoard() {

	// copy values to clipboard
	float fCR( float( m_color_cr.GetPos() - 50 ) / 50 );
	float fMG( float( m_color_mg.GetPos() - 50 ) / 50 );
	float fYB( float( m_color_yb.GetPos() - 50 ) / 50 );
	float fBri( float( m_brightness.GetPos() - 50 ) / 50 );
	float fCon( float( m_contrast.GetPos() - 50 ) / 25 );
	float fSat( float( m_saturation.GetPos() ) / 50 );
	DWORD dwDNR( m_dnr.GetPos() / 5 );
	
	// flip
	int iFlip( m_vflip.GetCurSel() );
	CString sFlip( L"auto" );
	switch( iFlip ) {
		case 1: sFlip = L"true"; break;
		case 2: sFlip = L"false"; break;
		case 0:;
		default:;
	}

	CString sClipboard;
	sClipboard.Format( L"input( \"\", \"%s\" ) {\n\tcolor_balance( %.2f, %.2f, %.2f );\n\tbrightness( %.2f );\n\tcontrast( %.2f );\n\tsaturation( %.2f );\n\tnoise_reduction( %i );\n\tvertical_flip( %s );\n}", m_sVideoPath.GetString(), fCR, fMG, fYB, fBri, fCon, fSat, dwDNR, sFlip.GetString() );
	OpenClipboard();
	EmptyClipboard();
	HGLOBAL hResult = GlobalAlloc( GMEM_MOVEABLE, sClipboard.GetLength() * 2 + 2 );
	if( hResult != NULL ) {
		LPTSTR lptstrCopy = ( LPTSTR ) GlobalLock( hResult );
		memset( lptstrCopy, 0, sClipboard.GetLength() * 2 + 2 );
		memcpy( lptstrCopy, sClipboard.GetString(), sClipboard.GetLength() * 2 );
		GlobalUnlock( hResult );
	}
	SetClipboardData( CF_UNICODETEXT, hResult );
	CloseClipboard();

	// reload video
	bool bReload = false;

	// DNR change
	if( dwDNR != m_dwDNR ) {
		m_dwDNR = dwDNR;
		bReload = true;
	}

	// flip change
	if( iFlip != m_iFlip ) {
		m_iFlip = iFlip;
		bReload = true;
	}

	// reload
	if( bReload )
		Load( m_sVideoPath );
}


// hscroll
void CAdjustmentsDlg::OnHScroll( UINT _nSBCode, UINT _nPos, CScrollBar * _pScrollBar ) {

	// copy to clipboard
	CopyToClipBoard();	
	CDialog::OnHScroll( _nSBCode, _nPos, _pScrollBar );
}


// vertical flip selection change
void CAdjustmentsDlg::OnCbnSelchangeCombo1() {

	// copy to clipboard
	CopyToClipBoard();	
}


// load video
void CAdjustmentsDlg::OnBnClickedLoad() {

	// load dialog
	CString sAllExt( _T("Video Files (*.mov)|*.mov;*.mpg;*.avi|All Files (*.*)|*.*||") );
	CFileDialog dlg( TRUE, NULL, NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, sAllExt.GetString() );
	if( dlg.DoModal() != IDOK )
		return;

	// reset sliders
	CSliderCtrl * sliders[ 6 ] = { &m_color_cr, &m_color_mg, &m_color_yb, &m_brightness, &m_contrast, &m_saturation };
	for( int i = 0; i < 6; i ++ )
		sliders[ i ]->SetPos( 50 );
	m_dnr.SetPos( 0 );
	m_vflip.SetCurSel( 0 );

	// load video
	Load( dlg.GetPathName() );
}


// load video
void CAdjustmentsDlg::Load( LPCTSTR _sVideoPath ) {

	// delete objects
	delete m_pFFMPEGCapture;
	m_pFFMPEGCapture = NULL;
	delete m_pInput;
	m_pInput = NULL;

	// vertical flip depends of sel
	CSIAdjustment::eFlip eVerticalFlip = CSIAdjustment::FT_AUTO;
	switch( m_vflip.GetCurSel() ) {
		case 1: eVerticalFlip = CSIAdjustment::FT_TRUE; break;
		case 2: eVerticalFlip = CSIAdjustment::FT_FALSE; break;
		case 0:;
		default:;
	}

	// create FFMPEG Capture
	m_sVideoPath = _sVideoPath;
	m_pInput = new CSInput( L"Adjustment", m_sVideoPath, CSInput::IF_CENTER, 0, 0 );
	m_pFFMPEGCapture = new CFFMPEGCapture( *m_pInput, 0, 100, m_rctDraw.Width(), m_rctDraw.Height(), 100, m_dwDNR, eVerticalFlip );
	if( !m_pFFMPEGCapture->StartCapture() ) {
		delete m_pFFMPEGCapture;
		m_pFFMPEGCapture = NULL;
		delete m_pInput;
		MessageBox( L"Failed to start capture on video.", L"Adjustments", IDOK | MB_ICONERROR );
	}
	m_dwPosition = 0;
}


// pre translate message
BOOL CAdjustmentsDlg::PreTranslateMessage( MSG * _pMSG ) {

	// paste
	if( _pMSG->message == WM_KEYDOWN && _pMSG->wParam == L'V' ) {

		// something available ?
		if( IsClipboardFormatAvailable( CF_UNICODETEXT ) ) {

			// get content
			CString sClipboard;
			OpenClipboard();
			HANDLE hData = GetClipboardData( CF_UNICODETEXT );
			HGLOBAL hResult = GlobalLock( hData );
			if( hResult != NULL )
				sClipboard = ( LPTSTR )hResult;
			GlobalUnlock( hData );
			CloseClipboard();

			// color_balance( %.2f, %.2f, %.2f );
			int iStart = 0;
			CString sParams;
			if( ExtractCommand( sClipboard, L"color_balance", sParams, iStart ) ) {
				int iParamStart = 0;
				CString sdCR, sdMG, sdYB;
				ExtractParam( sParams, sdCR, iParamStart );
				ExtractParam( sParams, sdMG, iParamStart );
				ExtractParam( sParams, sdYB, iParamStart );
				m_color_cr.SetPos( int( _wtof( sdCR ) * 50 + 50 ) );
				m_color_mg.SetPos( int( _wtof( sdMG ) * 50 + 50 ) );
				m_color_yb.SetPos( int( _wtof( sdYB ) * 50 + 50 ) );
			}

			// brightness( %.2f );
			iStart = 0;
			while( ExtractCommand( sClipboard, L"brightness", sParams, iStart ) ) {
				int iParamStart = 0;
				CString sdBrightness;
				ExtractParam( sParams, sdBrightness, iParamStart );
				m_brightness.SetPos( int( _wtof( sdBrightness ) * 50 + 50 ) );
			}

			// contrast( %.2f );
			iStart = 0;
			while( ExtractCommand( sClipboard, L"contrast", sParams, iStart ) ) {
				int iParamStart = 0;
				CString sdContrast;
				ExtractParam( sParams, sdContrast, iParamStart );
				m_contrast.SetPos( int( _wtof( sdContrast ) * 25 + 50 ) );
			}

			// saturation( %.2f );
			iStart = 0;
			while( ExtractCommand( sClipboard, L"saturation", sParams, iStart ) ) {
				int iParamStart = 0;
				CString sdSaturation;
				ExtractParam( sParams, sdSaturation, iParamStart );
				m_saturation.SetPos( int( _wtof( sdSaturation ) * 50 ) );
			}

			// reload video
			bool bReload = false;

			// noise_reduction( %i );
			iStart = 0;
			while( ExtractCommand( sClipboard, L"noise_reduction", sParams, iStart ) ) {
				int iParamStart = 0;
				CString sdLevel;
				ExtractParam( sParams, sdLevel, iParamStart );
				m_dwDNR = _wtoi( sdLevel );
				m_dnr.SetPos( m_dwDNR * 5 );
				bReload = true;
			}

			// vertical_flip( %s );
			iStart = 0;
			while( ExtractCommand( sClipboard, L"vertical_flip", sParams, iStart ) ) {
				int iParamStart = 0;
				CString sFlip;
				ExtractParam( sParams, sFlip, iParamStart );
				if( sFlip == L"auto" ) m_vflip.SetCurSel( 0 ); else
				if( sFlip == L"true" ) m_vflip.SetCurSel( 1 ); else
				if( sFlip == L"false" ) m_vflip.SetCurSel( 2 );
				bReload = true;
			}

			// reload video
			if( bReload )
				Load( m_sVideoPath );
		}
	}

	return CDialog::PreTranslateMessage( _pMSG );
}