// A. Sints
#include "stdafx.h"

#include "CtrlAutoPool.h"


// ctor
CCtrlAuto::CCtrlAuto() : m_iColCount( -1 ), m_pObj( NULL ), m_bIsListCtrl( false ) {

	m_originalRect = CRect( 0, 0, 0, 0 );
	m_currentRect = CRect( 0, 0, 0, 0 );
	m_originalWndSz = CSize( 0, 0 );
	memset( m_originalColWidth, 0, 255 );
	m_eTop = m_eLeft = m_eBottom = m_eRight = EA_FIXED;
}


// copy operator
CCtrlAuto & CCtrlAuto::operator=( const CCtrlAuto & _ref ) {

	m_eTop			= _ref.m_eTop;
	m_eLeft			= _ref.m_eLeft;
	m_eBottom		= _ref.m_eBottom;
	m_eRight		= _ref.m_eRight;
	m_pObj			= _ref.m_pObj;
	m_originalRect	= _ref.m_originalRect;
	m_currentRect	= _ref.m_currentRect;
	m_originalWndSz	= _ref.m_originalWndSz;

	// CListCtrl object
	m_bIsListCtrl	= _ref.m_bIsListCtrl;
	m_iColCount		= _ref.m_iColCount;
	memcpy( m_originalColWidth, _ref.m_originalColWidth, 255 );

	return *this;
}


// set object
void CCtrlAuto::SetObject( CWnd * _pObj,
						  eAnchor _eTop,
						  eAnchor _eLeft,
						  eAnchor _eBottom,
						  eAnchor _eRight ) {

	m_pObj = _pObj;

	// set anchors
	m_eTop = _eTop;
	m_eLeft = _eLeft;
	m_eBottom = _eBottom;
	m_eRight = _eRight;

	// object is a list?
	m_bIsListCtrl = dynamic_cast< CListCtrl * >( _pObj ) != NULL;
}


// build pos
void CCtrlAuto::Build( HWND _hWnd ) {

	if( m_pObj == NULL )
		return;

	// get curr CWnd
	CWnd * pWnd = CWnd::FromHandle( _hWnd );
	if( pWnd == NULL )
		return;

	// get original window size
	CRect rect;
	pWnd->GetClientRect( &rect );
	m_originalWndSz = CSize( rect.Width(), rect.Height() );	

	// get original object rect
	m_pObj->GetWindowRect( m_originalRect );
	pWnd->ScreenToClient( m_originalRect );
	m_currentRect = m_originalRect;

	// object is a list
	if( m_bIsListCtrl ) {

		// count columns
		LVCOLUMN colParam;
		colParam.mask = LVCF_WIDTH;
		m_iColCount = 0;
		while( ( ( CListCtrl * ) m_pObj )->GetColumn( m_iColCount++, &colParam ) == TRUE );
		m_iColCount--;

		// feed original column size
		for( int i = 0; i < m_iColCount; i++ )
			m_originalColWidth[ i ] = ( ( CListCtrl * ) m_pObj )->GetColumnWidth( i );
	}
}


// resize list
void CCtrlAuto::Resize( CSize _currentSz ) {

	if( m_pObj == NULL )
		return;

	// object is a list
	if( m_bIsListCtrl ) {
	
		// resize columns
		for( int i = 0; i < m_iColCount; i++ )
			( ( CListCtrl * ) m_pObj )->SetColumnWidth( i, m_originalColWidth[ i ] * _currentSz.cx / m_originalWndSz.cx );
	}

	// set current rect
	m_currentRect = m_originalRect;
	
	// top
	if( m_eTop == EA_FLOAT )			m_currentRect.top = m_currentRect.top * _currentSz.cy / m_originalWndSz.cy;
	// left
	if( m_eLeft == EA_FLOAT )			m_currentRect.left = m_currentRect.left * _currentSz.cx / m_originalWndSz.cx;
	
	// right
	if( m_eRight == EA_FIXED )			m_currentRect.right = _currentSz.cx - ( m_originalWndSz.cx - m_currentRect.right );
	else if( m_eRight == EA_FLOAT )		m_currentRect.right = m_currentRect.right * _currentSz.cx / m_originalWndSz.cx;
	else								m_currentRect.right = m_currentRect.left + m_originalRect.Width();
	// bottom
	if( m_eBottom == EA_FIXED )			m_currentRect.bottom = _currentSz.cy - ( m_originalWndSz.cy - m_currentRect.bottom );
	else if( m_eBottom == EA_FLOAT )	m_currentRect.bottom = m_currentRect.bottom * _currentSz.cy / m_originalWndSz.cy;
	else								m_currentRect.bottom = m_currentRect.top + m_originalRect.Height();
	
	// resize list itself
	m_pObj->MoveWindow( m_currentRect );
}


// top left limit
bool CCtrlAuto::TopLeftLimit( CPoint _tlL ) {

	if( m_pObj == NULL )
		return true;

	CRect currRectBefore( m_currentRect );

	// top limit
	if( _tlL.y != -1 && m_currentRect.top < _tlL.y ) {
		int iY = _tlL.y - m_currentRect.top;
		m_currentRect.top = _tlL.y;
		if( m_eBottom == EA_STATIC )
			m_currentRect.bottom += iY;
	}

	// left limit
	if( _tlL.x != -1 && m_currentRect.left < _tlL.x ) {
		int iX = _tlL.x - m_currentRect.left;
		m_currentRect.left = _tlL.x;
		if( m_eRight == EA_STATIC )
			m_currentRect.right += iX;
	}

	// resize list itself
	m_pObj->MoveWindow( m_currentRect );

	// true if already docked
	return( m_currentRect == currRectBefore ? true : false );
}


//----------------------------

// add object in pool
void CCtrlAutoPool::AddObject( CWnd * _pObj,
							  CCtrlAuto::eAnchor _eTop,
							  CCtrlAuto::eAnchor _eLeft,
							  CCtrlAuto::eAnchor _eBottom,
							  CCtrlAuto::eAnchor _eRight ) {

	CCtrlAuto ctrlAuto;
	ctrlAuto.SetObject( _pObj, _eTop, _eLeft, _eBottom, _eRight );
	m_vPool.AddTail( ctrlAuto );
}


// build pool objects pos
void CCtrlAutoPool::Build( HWND _hWnd ) {

	// build
	for( POSITION pos = m_vPool.GetHeadPosition(); pos; )
		m_vPool.GetNext( pos ).Build( _hWnd );
}


// resize pool objects
void CCtrlAutoPool::Resize( CSize _currentSz ) {
	
	// resize
	for( POSITION pos = m_vPool.GetHeadPosition(); pos; )
		m_vPool.GetNext( pos ).Resize( _currentSz );

	// we need to handle limits
	bool bStable = false;
	while( !bStable ) {
		bStable = true;
		for( POSITION pos = m_vPool.GetHeadPosition(); pos; ) {
			CCtrlAuto &obj = m_vPool.GetNext( pos );
			if( !obj.TopLeftLimit( _GetTopLeftLimit( obj ) ) )
				bStable = false;
		}
	}
}


// get top left limit of an object
CPoint CCtrlAutoPool::_GetTopLeftLimit( CCtrlAuto _obj ) {

	CPoint tlL( -1, -1 );
	CRect objORect = _obj.GetOriginalRect();
	for( POSITION pos = m_vPool.GetHeadPosition(); pos; ) {
		CCtrlAuto &currObj = m_vPool.GetNext( pos );
		CRect currObjORect = currObj.GetOriginalRect();
		CRect currObjCRect = currObj.GetCurrentRect();
		if( currObjORect == objORect )
			continue;

		// top limit
		if( objORect.top > currObjORect.bottom && currObjCRect.bottom > tlL.y )
			tlL.y = currObjCRect.bottom;

		// left limit
		if( objORect.left > currObjORect.right && currObjCRect.right > tlL.x )
			tlL.x = currObjCRect.right;
	}

	return tlL;
}


// get object
CCtrlAuto * CCtrlAutoPool::GetObject( CWnd * _pObj ) {

	for( POSITION pos = m_vPool.GetHeadPosition(); pos; ) {
		CCtrlAuto &currObj = m_vPool.GetNext( pos );
		if( currObj.GetObject() == _pObj )
			return &currObj;
	}

	// not found
	return NULL;
}
