// A. Sints
#include "stdafx.h"

#include "..\StukaDlg.h"
#include "AviWriter.h"


// vfw library
#pragma comment( lib, "vfw32.lib" )


// dtor
CAVIWriter::~CAVIWriter() {

	// close file
	if( m_pAVIFile != NULL )
		Close();
}


// create file
const bool CAVIWriter::Create( const bool _bReinit ) {

	// already created
	if( m_pAVIFile != NULL ) {
		AVIFileExit();
		CStukaDlg::Output( L"Error (AVIWriter): Failed to create AVIWriter: AVI file was already created.\n" );
		return false;
	}

	// init AVI File
	AVIFileInit();

	// remove previous file
	::DeleteFile( m_sFileName.GetString() );

	// open AVI file
	HRESULT hRes = AVIFileOpen( &m_pAVIFile, m_sFileName.GetString(), OF_WRITE | OF_CREATE, 0 );
	if( hRes != AVIERR_OK ) {
		AVIFileExit();
		CStukaDlg::Output( L"Error (AVIWriter): Failed to create AVIWriter: Failed to create AVI file.\n" );
		return false;
	}

	// joint an audio file
	if( !m_sAudioFileName.IsEmpty() ) {
	
		if( !_bReinit ) {
		
			// try to read audio file
			CFile fAudio;
			if( fAudio.Open( m_sAudioFileName.GetString(), CFile::modeRead ) == FALSE ) {
				CStukaDlg::Output( L"Error (Script): Failed to open wave file.\n" );
				return false;
			}

			// read file
			m_uiFileLen = UINT( fAudio.GetLength() );
			char * pBuffer = new char[ m_uiFileLen ];
			if( fAudio.Read( pBuffer, m_uiFileLen ) != m_uiFileLen ) {
				fAudio.Close();
				CStukaDlg::Output( L"Error (Script): Failed to read wave file.\n" );
				return false;
			}

			// close file
			fAudio.Close();

			// cast buffer as wave file
			m_pWaveData = ( WavChunk * ) pBuffer;
		}

		// set wave format
		WAVEFORMATEX wfx;
		wfx.wFormatTag		= m_pWaveData->fmt.wFormatTag;
		wfx.cbSize			= 0;
		wfx.nAvgBytesPerSec	= m_pWaveData->fmt.dwAvgBytesPerSec;
		wfx.nBlockAlign		= m_pWaveData->fmt.wBlockAlign;
		wfx.nChannels		= m_pWaveData->fmt.wChannels;
		wfx.nSamplesPerSec	= m_pWaveData->fmt.dwSamplesPerSec;
		wfx.wBitsPerSample	= m_pWaveData->fmt.wBitsPerSample;

		// create the stream
		AVISTREAMINFO ahdr;
		ZeroMemory( &ahdr, sizeof( ahdr ) );
		ahdr.fccType		= streamtypeAUDIO;
		ahdr.dwScale		= wfx.nBlockAlign;
		ahdr.dwRate			= wfx.nSamplesPerSec * wfx.nBlockAlign; 
		ahdr.dwSampleSize	= wfx.nBlockAlign;
		ahdr.dwQuality		= ( DWORD ) -1;

		// create audio stream
		hRes = AVIFileCreateStream( m_pAVIFile, &m_pAVIAudioStream, &ahdr );
		if( hRes != AVIERR_OK ) {
			CStukaDlg::Output( L"Error (AVIWriter): Failed to create AVIWriter: Failed to create AVI audio stream.\n" );
			return false;
		}

		// set stream format
		hRes = AVIStreamSetFormat( m_pAVIAudioStream, 0, &wfx, sizeof( WAVEFORMATEX ) );
		if( hRes != AVIERR_OK ) {
			CStukaDlg::Output( L"Error (AVIWriter): Failed to create AVIWriter: Failed to set audio stream format.\n" );
			return false;
		}

		// set sizes
		m_iSizeBytes = ahdr.dwRate / 25;
		m_iSizeSamples = m_iSizeBytes / wfx.nBlockAlign;
	}

	// create file stream info
	AVISTREAMINFO AVIStreamInfo;
	ZeroMemory( &AVIStreamInfo, sizeof( AVISTREAMINFO ) );
	AVIStreamInfo.fccType = streamtypeVIDEO;
	AVIStreamInfo.fccHandler = 0; 
	AVIStreamInfo.dwScale = 1000 / 25;
	AVIStreamInfo.dwRate = 1000;
	AVIStreamInfo.dwSuggestedBufferSize = m_dwWidth * m_dwHeight * 3; // RGB
	SetRect( &AVIStreamInfo.rcFrame, 0, 0, m_dwWidth, m_dwHeight );

	// create stream
	hRes = AVIFileCreateStream( m_pAVIFile, &m_pAVIStream, &AVIStreamInfo );
	if( hRes != AVIERR_OK ) {
		CStukaDlg::Output( L"Error (AVIWriter): Failed to create AVIWriter: Failed to create AVI stream.\n" );
		return false;
	}

	// create AVI compression options
	AVICOMPRESSOPTIONS AVICompressionOptions; 
	ZeroMemory( &AVICompressionOptions, sizeof( AVICOMPRESSOPTIONS ) );
	AVICompressionOptions.fccHandler = mmioFOURCC( 'D','I','B',' ' );

	// create un-compressed stream
	hRes = AVIMakeCompressedStream( &m_pAVICompressedStream, m_pAVIStream, &AVICompressionOptions, 0 );
	if( hRes != AVIERR_OK ) {
		CStukaDlg::Output( L"Error (AVIWriter): Failed to create AVIWriter: Failed to create compressed stream.\n" );
		return false;
	}

	// create bitmap info header	
	ZeroMemory( &m_bmpInfo, sizeof( BITMAPINFO ) );
	m_bmpInfo.bmiHeader.biSize = sizeof( BITMAPINFOHEADER );
	m_bmpInfo.bmiHeader.biBitCount = 24;
	m_bmpInfo.bmiHeader.biWidth = m_dwWidth;
	m_bmpInfo.bmiHeader.biHeight = m_dwHeight;
	m_bmpInfo.bmiHeader.biPlanes = 1;
	m_bmpInfo.bmiHeader.biSizeImage = m_dwHeight * m_dwWidth * 3;

	// create DIBSection
	if( !_bReinit )
		m_hBmp = CreateDIBSection( NULL, &m_bmpInfo, DIB_RGB_COLORS, ( void ** )&( m_lpBits ), NULL, 0 );

	// set stream format
	hRes = AVIStreamSetFormat( m_pAVICompressedStream, 0, &m_bmpInfo.bmiHeader, m_bmpInfo.bmiHeader.biSize );
	if( hRes != AVIERR_OK ) {
		CStukaDlg::Output( L"Error (AVIWriter): Failed to create AVIWriter: Failed to set stream format.\n" );
		return false;
	}

	// ok
	return true;
}


// close file
void CAVIWriter::Close( const bool _bReinit ) {

	// if audio
	if( !_bReinit && m_pWaveData != NULL ) {

		// delete audio buffer
		delete [] ( ( char * ) m_pWaveData );
		m_pWaveData = NULL;
	}

	// release audio AVI Stream
	AVIStreamRelease( m_pAVIAudioStream );
	m_pAVIAudioStream = NULL;

	// delete bitmap
	if( !_bReinit )
		DeleteObject( m_hBmp );

	// release compressed AVI Stream
	AVIStreamRelease( m_pAVICompressedStream );
	m_pAVICompressedStream = NULL;

	// release AVI Stream
	AVIStreamRelease( m_pAVIStream );
	m_pAVIStream = NULL;

	// close AVI file
	AVIFileClose( m_pAVIFile );
	m_pAVIFile = NULL;
	AVIFileExit();

	// rename last file if necessary
	if( !_bReinit && m_dwFileIndex != 0 ) {

		// rename
		int iPos = m_sFileName.ReverseFind( L'.' );
		CString sNewName;
		sNewName.Format( L"%s.%.2i%s", m_sFileName.Left( iPos ), m_dwFileIndex++, m_sFileName.Right( m_sFileName.GetLength() - iPos ) );
		if( MoveFileEx( m_sFileName.GetString(), sNewName.GetString(), MOVEFILE_REPLACE_EXISTING ) == FALSE )
			CStukaDlg::Output( L"Error (AVIWriter): Failed to rename AVI video.\n" );
	}
}


// add a frame
const bool CAVIWriter::InsertFrame( const DWORD _dwPosition ) {

	// no file created
	if( m_pAVIFile == NULL ) {
		CStukaDlg::Output( L"Error (AVIWriter): Failed to insert frame: No AVI file created.\n" );
		return false;
	}

	// get AVI size
	WIN32_FILE_ATTRIBUTE_DATA fileInfo;
	if( GetFileAttributesEx( m_sFileName.GetString(), GetFileExInfoStandard, ( void * ) & fileInfo ) == FALSE ) {
		CStukaDlg::Output( L"Error (AVIWriter): Failed to get AVI file info.\n" );
		return false;
	}
	
	// did we reach the size limit of 1Go?
	if( ( __int64( fileInfo.nFileSizeLow ) | ( __int64( fileInfo.nFileSizeHigh ) << 32 ) ) > 1024 * 1024 * 1024 ) {

		// close current AVI (reinit)
		Close( true );

		// rename
		int iPos = m_sFileName.ReverseFind( L'.' );
		CString sNewName;
		sNewName.Format( L"%s.%.2i%s", m_sFileName.Left( iPos ), m_dwFileIndex++, m_sFileName.Right( m_sFileName.GetLength() - iPos ) );
		if( MoveFileEx( m_sFileName.GetString(), sNewName.GetString(), MOVEFILE_REPLACE_EXISTING ) == FALSE ) {
			CStukaDlg::Output( L"Error (AVIWriter): Failed to rename AVI video.\n" );
			return false;
		}

		// decal start position
		m_dwStartPosition = _dwPosition - m_dwOffset;

		// create (reinit)
		if( !Create( true ) ) {
			CStukaDlg::Output( L"Error (AVIWriter): Failed to create next AVI video.\n" );
			return false;
		}
	}

	// write stream
	HRESULT hRes = AVIStreamWrite( m_pAVICompressedStream, _dwPosition - m_dwStartPosition - m_dwOffset, 1, m_lpBits, m_bmpInfo.bmiHeader.biSizeImage, AVIIF_KEYFRAME, 0, 0 );
	if( hRes != AVIERR_OK ) {
		CStukaDlg::Output( L"Error (AVIWriter): Failed to insert frame: Failed to write stream.\n" );
		return false;
	}

	// write audio stream
	if( m_pAVIAudioStream != NULL && ( ( ( _dwPosition * m_iSizeBytes ) + m_iSizeBytes ) < m_uiFileLen ) ) {
		hRes = AVIStreamWrite( m_pAVIAudioStream, ( _dwPosition - m_dwStartPosition - m_dwOffset ) * m_iSizeSamples, m_iSizeSamples, *( ( &m_pWaveData->dat.data ) + ( _dwPosition * m_iSizeBytes ) ), m_iSizeBytes, 0, NULL, NULL );
		if( hRes != AVIERR_OK ) {
			CStukaDlg::Output( L"Error (AVIWriter): Failed to insert frame: Failed to write audio stream.\n" );
			return false;
		}
	}

	// ok
	return true;
}