// A. Sints
#pragma once


// video
#include <vfw.h>


// simple AVI video writer class
class CAVIWriter sealed {
public:
	
	// dflt ctor
	CAVIWriter( LPCTSTR _sFileName, const DWORD _dwWidth, const DWORD _dwHeight, LPCTSTR _sAudioFileName = L"" )
		: m_sFileName( _sFileName ), m_dwWidth( _dwWidth ), m_dwHeight( _dwHeight ), m_sAudioFileName( _sAudioFileName ),
		m_pAVIFile( NULL ), m_pAVIStream( NULL ), m_pAVICompressedStream( NULL ), m_lpBits( NULL ), m_hBmp( NULL ),
		m_pWaveData( NULL ), m_pAVIAudioStream( NULL ), m_dwFileIndex( 0 ), m_dwStartPosition( 0 ), m_dwOffset( 0 ) {}

	// dtor
	~CAVIWriter();


public:

	// create file
	const bool Create( const bool _bReinit = false );

	// offset writer
	void Offset( const DWORD & _dwOffset ) { m_dwOffset = _dwOffset; }

	// close file
	void Close( const bool _bReinit = false );

	// add a frame
	const bool InsertFrame( const DWORD _dwPosition );

	// get bitmap
	const HBITMAP & GetBitmap() const { return m_hBmp; }


private:

	// filename
	const CString m_sFileName;

	// avi dimensions
	DWORD m_dwWidth;
	DWORD m_dwHeight;

	// audio filename
	const CString m_sAudioFileName;

	// offset
	DWORD m_dwOffset;


private:

	// AVI file
	IAVIFile * m_pAVIFile;

	// AVI Stream
	IAVIStream * m_pAVIStream;

	// AVI compressed stream
	IAVIStream * m_pAVICompressedStream;

	// bitmap info
	BITMAPINFO m_bmpInfo;

	// bitmap bits
	unsigned char * m_lpBits;

	// bitmap handle
	HBITMAP m_hBmp;

	// fmt chunk struct
	typedef struct {
		char id[ 4 ];					// "fmt "
		unsigned long size;				// 16
		short wFormatTag;				// WAVE_FORMAT_PCM=1
		unsigned short wChannels;		// 1 or 2 for mono or stereo
		unsigned long dwSamplesPerSec;	// 11025 or 22050 or 44100
		unsigned long dwAvgBytesPerSec;	// wBlockAlign * dwSamplesPerSec
		unsigned short wBlockAlign;		// wChannels * (wBitsPerSample==8?1:2)
		unsigned short wBitsPerSample;	// 8 or 16, for bits per sample
	} FmtChunk;

	// data struct
	typedef struct {
		char id[ 4 ];					// "data"
		unsigned long size;				// datsize, size of the following array
		unsigned char data[ 1 ];		// the raw data goes here
	} DataChunk;

	// wave chunk
	typedef struct {
		char id[ 4 ];					// "RIFF"
		unsigned long size;				// datsize+8+16+4
		char type[ 4 ];					// "WAVE"
		FmtChunk fmt;
		DataChunk dat;
	} WavChunk;

	// wave ptr
	WavChunk * m_pWaveData;

	// avi AUDIO stream
	IAVIStream * m_pAVIAudioStream;

	// audio file length
	UINT m_uiFileLen;

	// sizes
	int m_iSizeBytes;
	int m_iSizeSamples;

	// file index
	DWORD m_dwFileIndex;
	// start position
	DWORD m_dwStartPosition;
};