// A. Sints
#include "stdafx.h"

#include "..\StukaDlg.h"
#include "Extract.h"



// extract a script block
const bool CExtract::ExtractBlock( const CString & _sScript, LPCTSTR _sChunk, CString & _rsParams, CString & _rsContent, int & _riStart ) {

	// find chunk at given start position
	int iPos = _sScript.Find( _sChunk, _riStart );
	if( iPos == -1 )
		return false;

	// find "("
	int iParamFirst = _sScript.Find( L"(", iPos );
	if( iParamFirst == -1 ) {
		CStukaDlg::Output( L"Error (Script): Malformed script, missing '(' character.\n" );
		return false;
	}

	// find ")"
	int iParamLast = _sScript.Find( L")", iParamFirst );
	if( iParamLast == -1 ) {
		CStukaDlg::Output( L"Error (Script): Malformed script, missing ')' character.\n" );
		return false;
	}

	// extract params
	_rsParams = _sScript.Mid( iParamFirst + 1, iParamLast - iParamFirst - 1 );

	// find "{"
	int iContentFirst = _sScript.Find( L"{", iParamLast );
	if( iContentFirst == -1 ) {
		CStukaDlg::Output( L"Error (Script): Malformed script, missing '{' character.\n" );
		return false;
	}

	// find "}"
	_riStart = _sScript.Find( L"}", iContentFirst );
	if( _riStart == -1 ) {
		CStukaDlg::Output( L"Error (Script): Malformed script, missing '}' character.\n" );
		return false;
	}

	// extract content
	_rsContent = _sScript.Mid( iContentFirst + 1, _riStart - iContentFirst - 1 );

	// ok
	return true;
}


// extract a param
const bool CExtract::ExtractParam( const CString & _sParams, CString & _rsParam, int & _riStart, const bool _bSkipEOL ) {

	// search for the first non-useful (\n\r\t or space) character
	wchar_t c = _sParams.GetAt( _riStart );
	while( c == L' ' || c == L'\n' || c == L'\r' || c == L'\t' || c == L',' )
		c = _sParams.GetAt( ++_riStart );

	// eol
	if( c == 0 ) {
		if( !_bSkipEOL )
			CStukaDlg::Output( L"Error (Script): Malformed script.\n" );
		return false;
	}

	// this is a string
	if( c == L'\"' ) {
		int iLast = _sParams.Find( L'\"', _riStart + 1 );
		if( iLast == -1 ) {
			CStukaDlg::Output( L"Error (Script): Malformed script, missing '\"' character.\n" );
			return false;
		}

		// extract param
		_rsParam = _sParams.Mid( _riStart + 1, iLast - _riStart - 1 );
		_riStart = iLast + 1;
		return true;
	}
	// this is something else (value or type)
	else {
		int iLast1 = _sParams.Find( L',', _riStart + 1 );
		int iLast2 = _sParams.Find( L' ', _riStart + 1 );
		int iLast = ( iLast1 != -1 && iLast2 != -1 ) ? ( ( iLast1 < iLast2 ) ? iLast1 : iLast2 ) : ( iLast1 != -1 ? iLast1 : iLast2 );

		// end of param line
		if( iLast == -1 ) {
			_rsParam = _sParams.Right( _riStart );
			_riStart = -1;
			return true;
		}

		// extract param
		_rsParam = _sParams.Mid( _riStart, iLast - _riStart );
		_riStart = iLast + 1;
		return true;
	}
}


// extract a command
const bool CExtract::ExtractCommand( const CString & _sScript, LPCTSTR _sChunk, CString & _rsParams, int & _riStart ) {

	// find chunk at given start position
	int iPos = _sScript.Find( _sChunk, _riStart );
	if( iPos == -1 )
		return false;

	// find "("
	int iParamFirst = _sScript.Find( L"(", iPos );
	if( iParamFirst == -1 ) {
		CStukaDlg::Output( L"Error (Script): Malformed script, missing '(' character.\n" );
		return false;
	}

	// find ")"
	_riStart = _sScript.Find( L")", iParamFirst );
	if( _riStart == -1 ) {
		CStukaDlg::Output( L"Error (Script): Malformed script, missing ')' character.\n" );
		return false;
	}

	// extract params
	_rsParams = _sScript.Mid( iParamFirst + 1, _riStart - iParamFirst - 1 );

	// ok
	return true;
}