// A. Sints
#pragma once


// thread
#include "Thread.h"

// ffws
class CSInput;


// adjustment
#include "SIAdjustment.h"


// FFMPEG video capture class
class CFFMPEGCapture sealed : public CThread {
public:

	// dflt ctor
	CFFMPEGCapture( const CSInput & _rInput, const DWORD _dwFrameSeekMs, const DWORD _dwNbFrameToCapture, const DWORD _dwOutWidth, const DWORD _dwOutHeight, const DWORD _dwDisplayFrame, const DWORD _dwDNNLevel = 0, const CSIAdjustment::eFlip _eVerticalFlip = CSIAdjustment::FT_AUTO );

	// dtor
	~CFFMPEGCapture();


public:

	// start capture
	const bool StartCapture();

	// capture error
	const bool CaptureError() const { return m_bCaptureError; }

	// get bitmap
	const HBITMAP & GetBitmap( DWORD _dwPosition );

	// release a given bitmap
	void ReleaseBitmap( const DWORD _dwPosition );

	// set capture rect (used if display frame = 0)
	void SetCaptureRect( const CRect & _rctCapture ) { m_rctCapture = _rctCapture; }

	// set interpolation type
	static void SetInterpolationType( const int _iInterpolationType ) { m_iInterpolationType = _iInterpolationType; }


private:

	// callback
	const bool Callback();


private:

	// capture frames
	const bool _Capture();

	// compute cropping rectangle
	CRect _ComputeCropRect( const DWORD _dwSrcWidth, const DWORD _dwSrcHeight );

	// apply some dynamic noise reduction to a given bitmap
	void _DynamicNoiseReduction( const unsigned char * _pu8Src, HBITMAP & _hDstBmp, const DWORD _dwDNNLevel = 6 );


private:

	// seek from the video file start in millisecond
	const DWORD m_dwFrameSeekMs;

	// number of frame to capture
	const DWORD m_dwNbFrameToCapture;

	// output dimensions;
	const DWORD m_dwOutWidth, m_dwOutHeight;

	// input ref
	const CSInput & m_rInput;

	// display frame
	const DWORD m_dwDisplayFrame;

	// dynamic noise reduction level
	mutable DWORD m_dwDNRLevel;

	// vertical flip
	const CSIAdjustment::eFlip m_eVerticalFlip;


private:

	// interpolation type
	static int m_iInterpolationType;

	// number of image to buffer in advance
	static const DWORD m_dwImageBuffering;

	// error during capture
	bool m_bCaptureError;

	// current frame
	DWORD m_dwCurrentFrame;

	// null HBITMAP
	const HBITMAP m_hBmpNULL;

	// HBITMAP map
	CMutex m_mtxBmpMap;
	CMap< DWORD, DWORD, HBITMAP, HBITMAP > m_mBmp;

	// started
	CEvent m_evtStarted;

	// capture rect
	CRect m_rctCapture;

	// buffer for dynamic noise reduction
	unsigned char * m_pu8DNRBuffer;
	unsigned char m_divTable[ 766 ];

	// last get bitmap
	DWORD m_dwLastGetBitmap;
};