// A. Sints
#include "stdafx.h"

#include "..\StukaDlg.h"
#include "Layout.h"



// get layout type from string
const CLayout::eType CLayout::GetLayoutType( const CString & _sLayout ) {

	if( _sLayout == L"1fullscreen" )	return LT_1FULLSCREEN; else
	if( _sLayout == L"1left_1right" )	return LT_1LEFT1RIGHT; else
	if( _sLayout == L"1top_1bottom" )	return LT_1TOP1BOTTOM; else
	if( _sLayout == L"1left_2right" )	return LT_1LEFT2RIGHT; else
	if( _sLayout == L"2left_1right" )	return LT_2LEFT1RIGHT; else
	if( _sLayout == L"1top_2bottom" )	return LT_1TOP2BOTTOM; else
	if( _sLayout == L"2top_1bottom" )	return LT_2TOP1BOTTOM; else
	if( _sLayout == L"2top_2bottom" )	return LT_2TOP2BOTTOM; else
	if( _sLayout == L"1top_3bottom" )	return LT_1TOP3BOTTOM; else
	if( _sLayout == L"3top_1bottom" )	return LT_3TOP1BOTTOM; else
	if( _sLayout == L"2top_3bottom" )	return LT_2TOP3BOTTOM; else
	if( _sLayout == L"3top_2bottom" )	return LT_3TOP2BOTTOM; else
	if( _sLayout == L"3top_3bottom" )	return LT_3TOP3BOTTOM; else
	if( _sLayout == L"none" )	return LT_NONE; else	{
		CStukaDlg::Output( L"Error (Layout): Unknown layout type '" );
		CStukaDlg::Output( _sLayout );
		CStukaDlg::Output( L"'.\n" );
		return LT_NULL;
	}
}


// get layout rect list
const bool CLayout::GetLayout( const eType _eType, CList< CRect > & _rlRects, CList< CRect > & _rlLayoutLines ) {

	// reset list
	_rlRects.RemoveAll();
	_rlLayoutLines.RemoveAll();

	// computations
	DWORD dwWidthD2 = m_dwWidth >> 1;
	DWORD dwHeightD2 = m_dwHeight >> 1;
	DWORD dwWidthD31 = m_dwWidth / 3;
	DWORD dwWidthD32 = ( m_dwWidth << 1 ) / 3;

	// compute layout depending of the
	switch( _eType ) {

		// 1fullscreen
		case LT_1FULLSCREEN:
			_rlRects.AddTail( CRect( 0, 0, m_dwWidth, m_dwHeight ) );
			break;

		// 1left_1right
		case LT_1LEFT1RIGHT:
			_rlRects.AddTail( CRect( 0, 0, dwWidthD2, m_dwHeight ) );
			_rlRects.AddTail( CRect( dwWidthD2, 0, m_dwWidth, m_dwHeight ) );
			_rlLayoutLines.AddTail( CRect( dwWidthD2, 0, dwWidthD2, m_dwHeight ) );
			break;

		// 1top_1bottom
		case LT_1TOP1BOTTOM:
			_rlRects.AddTail( CRect( 0, dwHeightD2, m_dwWidth, m_dwHeight ) );
			_rlRects.AddTail( CRect( 0, 0, m_dwWidth, dwHeightD2 ) );
			_rlLayoutLines.AddTail( CRect( 0, dwHeightD2, m_dwWidth, dwHeightD2 ) );
			break; 

		// 1left_2right
		case LT_1LEFT2RIGHT:
			_rlRects.AddTail( CRect( 0, 0, dwWidthD2, m_dwHeight ) );
			_rlRects.AddTail( CRect( dwWidthD2, dwHeightD2, m_dwWidth, m_dwHeight ) );
			_rlRects.AddTail( CRect( dwWidthD2, 0, m_dwWidth, dwHeightD2 ) );
			_rlLayoutLines.AddTail( CRect( dwWidthD2, 0, dwWidthD2, m_dwHeight ) );
			_rlLayoutLines.AddTail( CRect( dwWidthD2, dwHeightD2, m_dwWidth, dwHeightD2 ) );
			break;

		// 2left_1right
		case LT_2LEFT1RIGHT:
			_rlRects.AddTail( CRect( 0, dwHeightD2, dwWidthD2, m_dwHeight ) );
			_rlRects.AddTail( CRect( 0, 0, dwWidthD2, dwHeightD2 ) );
			_rlRects.AddTail( CRect( dwWidthD2, 0, m_dwWidth, m_dwHeight ) );
			_rlLayoutLines.AddTail( CRect( dwWidthD2, 0, dwWidthD2, m_dwHeight ) );
			_rlLayoutLines.AddTail( CRect( 0, dwHeightD2, dwWidthD2, dwHeightD2 ) );
			break;

		// 1top_2bottom
		case LT_1TOP2BOTTOM:
			_rlRects.AddTail( CRect( 0, dwHeightD2, m_dwWidth, m_dwHeight ) );
			_rlRects.AddTail( CRect( 0, 0, dwWidthD2, dwHeightD2 ) );
			_rlRects.AddTail( CRect( dwWidthD2, 0, m_dwWidth, dwHeightD2 ) );
			_rlLayoutLines.AddTail( CRect( 0, dwHeightD2, m_dwWidth, dwHeightD2 ) );
			_rlLayoutLines.AddTail( CRect( dwWidthD2, 0, dwWidthD2, dwHeightD2 ) );
			break;

		// 2top_1bottom
		case LT_2TOP1BOTTOM:
			_rlRects.AddTail( CRect( 0, dwHeightD2, dwWidthD2, m_dwHeight ) );
			_rlRects.AddTail( CRect( dwWidthD2, dwHeightD2, m_dwWidth, m_dwHeight ) );
			_rlRects.AddTail( CRect( 0, 0, m_dwWidth, dwHeightD2 ) );
			_rlLayoutLines.AddTail( CRect( dwWidthD2, dwHeightD2, dwWidthD2, m_dwHeight ) );
			_rlLayoutLines.AddTail( CRect( 0, dwHeightD2, m_dwWidth, dwHeightD2 ) );			
			break;

		// 2top_2bottom
		case LT_2TOP2BOTTOM:
			_rlRects.AddTail( CRect( 0, dwHeightD2, dwWidthD2, m_dwHeight ) );
			_rlRects.AddTail( CRect( dwWidthD2, dwHeightD2, m_dwWidth, m_dwHeight ) );
			_rlRects.AddTail( CRect( 0, 0, dwWidthD2, dwHeightD2 ) );
			_rlRects.AddTail( CRect( dwWidthD2, 0, m_dwWidth, dwHeightD2 ) );
			_rlLayoutLines.AddTail( CRect( dwWidthD2, 0, dwWidthD2, m_dwHeight ) );
			_rlLayoutLines.AddTail( CRect( 0, dwHeightD2, m_dwWidth, dwHeightD2 ) );
			break;

		// 1top_3bottom
		case LT_1TOP3BOTTOM:
			_rlRects.AddTail( CRect( 0, dwHeightD2, m_dwWidth, m_dwHeight ) );
			_rlRects.AddTail( CRect( 0, 0, dwWidthD31 + 1, dwHeightD2 ) );
			_rlRects.AddTail( CRect( dwWidthD31, 0, dwWidthD32 + 1, dwHeightD2 ) );
			_rlRects.AddTail( CRect( dwWidthD32, 0, m_dwWidth, dwHeightD2 ) );
			_rlLayoutLines.AddTail( CRect( dwWidthD31, 0, dwWidthD31, dwHeightD2 ) );
			_rlLayoutLines.AddTail( CRect( dwWidthD32, 0, dwWidthD32, dwHeightD2 ) );
			_rlLayoutLines.AddTail( CRect( 0, dwHeightD2, m_dwWidth, dwHeightD2 ) );
			break;

		// 3top_1bottom
		case LT_3TOP1BOTTOM:
			_rlRects.AddTail( CRect( 0, dwHeightD2, dwWidthD31 + 1, m_dwHeight ) );
			_rlRects.AddTail( CRect( dwWidthD31, dwHeightD2, dwWidthD32 + 1, m_dwHeight ) );
			_rlRects.AddTail( CRect( dwWidthD32, dwHeightD2, m_dwWidth, m_dwHeight ) );
			_rlRects.AddTail( CRect( 0, 0, m_dwWidth, dwHeightD2 ) );
			_rlLayoutLines.AddTail( CRect( 0, dwHeightD2, m_dwWidth, dwHeightD2 ) );
			_rlLayoutLines.AddTail( CRect( dwWidthD31, dwHeightD2, dwWidthD31, m_dwHeight ) );
			_rlLayoutLines.AddTail( CRect( dwWidthD32, dwHeightD2, dwWidthD32, m_dwHeight ) );
			break;

		// 2top_3bottom
		case LT_2TOP3BOTTOM:
			_rlRects.AddTail( CRect( 0, dwHeightD2, dwWidthD2, m_dwHeight ) );
			_rlRects.AddTail( CRect( dwWidthD2, dwHeightD2, m_dwWidth, m_dwHeight ) );
			_rlRects.AddTail( CRect( 0, 0, dwWidthD31 + 1, dwHeightD2 ) );
			_rlRects.AddTail( CRect( dwWidthD31, 0, dwWidthD32 + 1, dwHeightD2 ) );
			_rlRects.AddTail( CRect( dwWidthD32, 0, m_dwWidth, dwHeightD2 ) );
			_rlLayoutLines.AddTail( CRect( dwWidthD2, dwHeightD2, dwWidthD2, m_dwHeight ) );
			_rlLayoutLines.AddTail( CRect( dwWidthD31, 0, dwWidthD31, dwHeightD2 ) );
			_rlLayoutLines.AddTail( CRect( dwWidthD32, 0, dwWidthD32, dwHeightD2 ) );
			_rlLayoutLines.AddTail( CRect( 0, dwHeightD2, m_dwWidth, dwHeightD2 ) );
			break;

		// 3top_2bottom
		case LT_3TOP2BOTTOM:
			_rlRects.AddTail( CRect( 0, dwHeightD2, dwWidthD31 + 1, m_dwHeight ) );
			_rlRects.AddTail( CRect( dwWidthD31, dwHeightD2, dwWidthD32 + 1, m_dwHeight ) );
			_rlRects.AddTail( CRect( dwWidthD32, dwHeightD2, m_dwWidth, m_dwHeight ) );
			_rlRects.AddTail( CRect( 0, 0, dwWidthD2, dwHeightD2 ) );
			_rlRects.AddTail( CRect( dwWidthD2, 0, m_dwWidth, dwHeightD2 ) );
			_rlLayoutLines.AddTail( CRect( dwWidthD31, dwHeightD2, dwWidthD31, m_dwHeight ) );
			_rlLayoutLines.AddTail( CRect( dwWidthD32, dwHeightD2, dwWidthD32, m_dwHeight ) );
			_rlLayoutLines.AddTail( CRect( dwWidthD2, 0, dwWidthD2, m_dwHeight ) );
			_rlLayoutLines.AddTail( CRect( 0, dwHeightD2, m_dwWidth, dwHeightD2 ) );
			break;

		// 3top_3bottom
		case LT_3TOP3BOTTOM:
			_rlRects.AddTail( CRect( 0, dwHeightD2, dwWidthD31 + 1, m_dwHeight ) );
			_rlRects.AddTail( CRect( dwWidthD31, dwHeightD2, dwWidthD32 + 1, m_dwHeight ) );
			_rlRects.AddTail( CRect( dwWidthD32, dwHeightD2, m_dwWidth, m_dwHeight ) );
			_rlRects.AddTail( CRect( 0, 0, dwWidthD31 + 1, dwHeightD2 ) );
			_rlRects.AddTail( CRect( dwWidthD31, 0, dwWidthD32 + 1, dwHeightD2 ) );
			_rlRects.AddTail( CRect( dwWidthD32, 0, m_dwWidth, dwHeightD2 ) );
			_rlLayoutLines.AddTail( CRect( dwWidthD31, 0, dwWidthD31, m_dwHeight ) );
			_rlLayoutLines.AddTail( CRect( dwWidthD32, 0, dwWidthD32, m_dwHeight ) );
			_rlLayoutLines.AddTail( CRect( 0, dwHeightD2, m_dwWidth, dwHeightD2 ) );
			break;		

		// wtf ?
		default:
			return false;
	}

	// ok
	return true;
}