// A. Sints
#pragma once


/* layouts:
 ___________    ___________    ___________    ___________    ___________    ___________    ___________ 
|           |  |     |     |  |     1     |  |     |  2  |  |  1  |     |  |     1     |  |  1  |  2  |
|     1     |  |  1  |  2  |  |-----------|  |  1  |-----|  |-----|  3  |  |-----------|  |-----------|
|           |  |     |     |  |     2     |  |     |  3  |  |  2  |     |  |  2  |  3  |  |     3     |
 -----------    -----------    -----------    -----------    -----------    -----------    ----------- 
 1fullscreen    1left_1right   1top_1bottom   1left_2right   2left_1right   1top_2bottom   2top_1bottom

 ___________    ___________    ___________    ___________    ___________    ___________ 
|  1  |  2  |  |     1     |  | 1 | 2 | 3 |  |  1  |  2  |  | 1 | 2 | 3 |  | 1 | 2 | 3 |
|-----|-----|  |-----------|  |-----------|  |-----------|  |-----------|  |-----------|
|  3  |  4  |  | 2 | 3 | 4 |  |     4     |  | 3 | 4 | 5 |  |  4  |  5  |  | 4 | 5 | 6 |
 -----------    -----------    -----------    -----------    -----------    ----------- 
 2top_2bottom   1top_3bottom   3top_1bottom   2top_3bottom   3top_2bottom   3top_3bottom

*/


// Layout class
class CLayout sealed {
public:

	// ctor
	CLayout( const DWORD _dwWidth, const DWORD _dwHeight )
		: m_dwWidth( _dwWidth ), m_dwHeight( _dwHeight ) {}

	// dtor
	~CLayout() {}


public:

	// layout type
	enum eType{ LT_NULL, LT_NONE, LT_1FULLSCREEN, LT_1LEFT1RIGHT, LT_1TOP1BOTTOM, LT_1LEFT2RIGHT, LT_2LEFT1RIGHT, LT_1TOP2BOTTOM, LT_2TOP1BOTTOM,
		LT_2TOP2BOTTOM, LT_1TOP3BOTTOM, LT_3TOP1BOTTOM, LT_2TOP3BOTTOM, LT_3TOP2BOTTOM, LT_3TOP3BOTTOM };

	// get rect
	CRect GetRect() { return CRect( 0, 0, m_dwWidth, m_dwHeight ); }


public:

	// get layout type from string
	static const eType GetLayoutType( const CString & _sLayout );

	// get layout rect list
	const bool GetLayout( const eType _eType, CList< CRect > & _rlRects, CList< CRect > & _rlLayoutLines );


private:

	// output dimensions
	const DWORD m_dwWidth, m_dwHeight;
};