// A. Sints
#pragma once


// CList with sort
template< class TYPE, class ARG_TYPE = const TYPE& >
class CListSort : public CList< TYPE, ARG_TYPE > {
public:

	// sort method (merge)
	// O(n log n)
	template< class SORTFUNC >
	void MergeSort( SORTFUNC _SortFunc ) {

		// just call recursive sub method
		m_pNodeHead = _MergeSort( _SortFunc, m_pNodeHead );
	}


	// sort method (bubble)
	// O(n)
	template< class SORTFUNC >
	void BubbleSort( SORTFUNC _SortFunc ) {

		// nothing to sort
		if( m_nCount <= 1 )
			return;

		bool bChanged = true;
		while( bChanged ) {
			bChanged = false;

			// start from head
			CNode * pNodeB = m_pNodeHead;
			while( pNodeB != NULL && pNodeB->pNext != NULL ) {

				CNode * pNodeA = pNodeB;
				pNodeB = pNodeA->pNext;

				// call the sort function to known if we need to swap nodes
				if( _SortFunc( pNodeB->data, pNodeA->data ) ) {

					// swap nodes
					pNodeB->pPrev = pNodeA->pPrev;
					if( pNodeB->pPrev != NULL )
						pNodeB->pPrev->pNext = pNodeB;
					pNodeA->pNext = pNodeB->pNext;
					if( pNodeA->pNext != NULL )
						pNodeA->pNext->pPrev = pNodeA;
					pNodeB->pNext = pNodeA;
					pNodeA->pPrev = pNodeB;

					// take care of head/tail
					if( pNodeB->pPrev == NULL )
						m_pNodeHead = pNodeB;
					if( pNodeA->pNext == NULL )
						m_pNodeTail = pNodeA;

					// jump to next node
					pNodeB = pNodeA;
					bChanged = true;
				}
			}
		}
	}


private:

	// sort method (merge) - recursive
	template< class SORTFUNC >
	CNode * _MergeSort( SORTFUNC _SortFunc, CNode * _pNodeA ) {

		// nothing to sort
		if( _pNodeA == NULL || _pNodeA->pNext == NULL )
			return _pNodeA;

		// B need to point halfway through the list
		CNode * pNodeB = _pNodeA;
		CNode * pTemp = _pNodeA;
		while( pTemp && pTemp->pNext ) {
			pNodeB = pNodeB->pNext;
			pTemp = pTemp->pNext->pNext;
		}

		// break the list in two
		pNodeB->pPrev->pNext = NULL;
		
		// recurse on the two sub lists
		_pNodeA = _MergeSort( _SortFunc, _pNodeA );
		pNodeB = _MergeSort( _SortFunc, pNodeB );

		// head, next & tail
		CNode * pNodeHead = NULL;
		CNode * pNodeNext = NULL;
		CNode * pNodeTail = NULL;

		// for each element to sort
		while( _pNodeA != NULL || pNodeB != NULL) {

			if( pNodeB == NULL ) { pNodeNext = _pNodeA; _pNodeA = _pNodeA->pNext; } else
			if( _pNodeA == NULL ) { pNodeNext = pNodeB; pNodeB = pNodeB->pNext; } else

			// call the sort function to known if we need to swap nodes
			if( _SortFunc( pNodeB->data, _pNodeA->data ) ) {
				pNodeNext = pNodeB;
				pNodeB = pNodeB->pNext;
			}
			else { pNodeNext = _pNodeA; _pNodeA = _pNodeA->pNext; }

			// set result
			if( pNodeHead == NULL )	pNodeHead = pNodeNext;
			else					pNodeTail->pNext = pNodeNext;

			// next
			pNodeNext->pPrev = pNodeTail;
			pNodeTail = pNodeNext;

			// take care of tail node
			if( pNodeTail->pNext == NULL )
				m_pNodeTail = pNodeTail;
		}

		// return head result
		return pNodeHead;
	}
};