// A. Sints
#include "stdafx.h"

#include "RGBAdjustment.h"

// sin
#include <math.h>


// dtor
CRGBAdjustment::~CRGBAdjustment() {

	// reset table
	ResetTable();
}


// reset table
void CRGBAdjustment::ResetTable() const {

	// release RGB adjustment table
	if( m_pRGBAdjust != NULL ) {
		delete [] m_pRGBAdjust;
		m_pRGBAdjust = NULL;
	}
}


// set color balance
void CRGBAdjustment::ColorBalance( const double _dCR, const double _dMG, const double _dYB ) {

	m_dColorBalCR = _dCR;
	m_dColorBalMG = _dMG;
	m_dColorBalYB = _dYB;
}


// build adjustment table
void CRGBAdjustment::_Build() const {

	// color balance
	unsigned char colorBalTable[ 3 ][ 256 ];
	double dR, dG, dB;
	double dRatio;
	for( int i = 0; i < 256; i++ ) {
		dRatio = 127 * sin( i * 0.012319971190548208778284876012861 ); // PI / 255
		dR = i + ( dRatio * m_dColorBalCR );
		colorBalTable[ 0 ][ i ] = dR > 255 ? 255 : ( dR < 0 ? 0 : unsigned char( dR ) );
		dG = i + ( dRatio * m_dColorBalMG );
		colorBalTable[ 1 ][ i ] = dG > 255 ? 255 : ( dG < 0 ? 0 : unsigned char( dG ) );
		dB = i + ( dRatio * m_dColorBalYB );
		colorBalTable[ 2 ][ i ] = dB > 255 ? 255 : ( dB < 0 ? 0 : unsigned char( dB ) );
	}

	// brightness
	unsigned char brightnessTable[ 256 ];
	double dBrightness;
	for( int i = 0; i < 256; i++ ) {
		dBrightness = i + ( ( 255 - i ) * m_dBrightnessRatio );
		brightnessTable[ i ] = dBrightness > 255 ? 255 : ( dBrightness < 0 ? 0 : unsigned char( dBrightness ) );
	}

	// contrast
	double contrastAdj[ 256 ];
	for( int i = 128; i < 256; i++ ) {
		contrastAdj[ i ] = ( 128 + ( sin( ( i - 128 ) * 0.012368475014133044245915918831809 ) * 127 ) ) - i; // ( PI / 2 ) / 127
		contrastAdj[ 255 - i ] = -contrastAdj[ i ];
	}

	unsigned char contrastTable[ 256 ];
	double dContrast;
	for( int i = 0; i < 256; i++ ) {
		dContrast = i + ( contrastAdj[ i ] * m_dContrastRatio );
		contrastTable[ i ] = i < 128 ? ( dContrast < 0 ? 0 : ( dContrast > 127 ? 127 : unsigned char( dContrast ) ) ) : ( dContrast < 127 ? 127 : ( dContrast > 255 ? 255 : unsigned char( dContrast ) ) );
	}

	// saturation
	unsigned char saturationTable[ 256 ][ 256 ];
	double dSaturation;
	for( int i = 0; i < 256; i++ ) {
		for( int j = 0; j < 256; j++ ) {
			dSaturation = j + int( double( i - j ) * m_dSaturationRatio );
			saturationTable[ i ][ j ] = dSaturation > 255 ? 255 : ( dSaturation < 0 ? 0 : unsigned char( dSaturation ) );
		}
	}

	// build all RGB adjustments table
	m_pRGBAdjust = new unsigned char[ 256 * 256 * 256 * 3 ];
	int iOffR, iOffRG, iOffRGB3;
	int iRcb, iGcb, iBcb, iRGBc;
	for( int R = 0; R < 256; R ++ ) {
		iOffR = R << 16;
		for( int G = 0; G < 256; G ++ ) {
			iOffRG = iOffR | ( G << 8 );
			for( int B = 0; B < 256; B ++ ) {
				// apply color balance, brightness and contrast
				iRcb = contrastTable[ brightnessTable[ colorBalTable[ 0 ][ R ] ] ];
				iGcb = contrastTable[ brightnessTable[ colorBalTable[ 1 ][ G ] ] ];
				iBcb = contrastTable[ brightnessTable[ colorBalTable[ 2 ][ B ] ] ];
				iRGBc = ( iRcb + iGcb + iBcb ) / 3;
				// apply saturation and set result
				iOffRGB3 = ( iOffRG | B ) * 3;
				m_pRGBAdjust[ iOffRGB3 + 2 ]	= saturationTable[ iRcb ][ iRGBc ];
				m_pRGBAdjust[ iOffRGB3 + 1 ]	= saturationTable[ iGcb ][ iRGBc ];
				m_pRGBAdjust[ iOffRGB3 ]		= saturationTable[ iBcb ][ iRGBc ];
			}
		}
	}
}