// A. Sints
#pragma once

// SI Adjustment
#include "SIAdjustment.h"


// Stuka RGB Adjustment class
class CRGBAdjustment sealed {
friend class CProject;
friend class CViewport;
public:

	// dflt ctor
	CRGBAdjustment()
		: m_dColorBalCR( 0 ), m_dColorBalMG( 0 ), m_dColorBalYB( 0 ),
		m_dBrightnessRatio( 0 ), m_dContrastRatio( 0 ), m_dSaturationRatio( 1 ),
		m_dwDNRLevel( 0 ), m_eFlip( CSIAdjustment::FT_AUTO ),
		m_pRGBAdjust( NULL ) {}

	// dtor
	~CRGBAdjustment();


public:

	// set color balance
	void ColorBalance( const double _dCR = 0, const double _dMG = 0, const double _dYB = 0 );

	// set brightness
	void Brightness( const double _dBrightnessRatio = 0 ) { m_dBrightnessRatio = _dBrightnessRatio; }

	// set contrast
	void Contrast( const double _dContrastRatio = 0 ) { m_dContrastRatio = _dContrastRatio; }

	// set saturation
	void Saturation( const double _dSaturationRatio = 1 ) { m_dSaturationRatio = _dSaturationRatio; }

	// set DNR level
	void SetDNRLevel( const DWORD _dwDNRLevel = 0 ) { m_dwDNRLevel = _dwDNRLevel; }

	// set vertical flip type
	void SetVerticalFlip( const CSIAdjustment::eFlip _eFlip ) { m_eFlip = _eFlip; }


private:

	// build adjustment table
	void _Build() const;

	// get table
	const unsigned char * const GetTable() const { if( m_pRGBAdjust == NULL ) _Build(); return m_pRGBAdjust; }

	// reset table
	void ResetTable() const;

	// get DNR level
	const DWORD & GetDNRLevel() const { return m_dwDNRLevel; }

	// get vertical flip
	const CSIAdjustment::eFlip & GetVerticalFlip() const { return m_eFlip; }


private:

	// color balance
	double m_dColorBalCR;
	double m_dColorBalMG;
	double m_dColorBalYB;

	// brightness
	double m_dBrightnessRatio;

	// contrast
	double m_dContrastRatio;

	// saturation
	double m_dSaturationRatio;

	// adjustment table
	mutable unsigned char * m_pRGBAdjust;

	// DNR level
	DWORD m_dwDNRLevel;

	// vertical flip
	CSIAdjustment::eFlip m_eFlip;
};