// A. Sints
#pragma once


/*
color_balance( CR, MG, YB );
brightness( brightness );
contrast( contrast );
saturation( saturation );
noise_reduction( level );
vertical_flip( flip ); // auto, true, false
*/

// RGB adjustment fwd
class CRGBAdjustment;


//------------------
// Stuka Script Input Adjustment class
class CSIAdjustment abstract {
public:
	CSIAdjustment() {}
	virtual ~CSIAdjustment() {}


public:

	// vertical flip type
	enum eFlip {
		FT_NULL,
		FT_AUTO,
		FT_TRUE,
		FT_FALSE };


public:

	// adjust RGB table
	virtual void Adjust( CRGBAdjustment & _rRGBAdjustment ) = 0;

	// get vertical flip type from string
	static const eFlip GetFlipType( const CString & _sFlip );
};


//------------------
// Stuka Script Input Adjustment ColorBalance class
class CSIAColorBalance sealed : public CSIAdjustment {
public:

	// dflt ctor
	CSIAColorBalance( const double _dCR, const double _dMG, const double _dYB )
		: m_dCR( _dCR ), m_dMG( _dMG ), m_dYB( _dYB ) {}


private:

	// adjust RGB table
	void Adjust( CRGBAdjustment & _rRGBAdjustment );


private:

	// Cyan/Red | Magenta/Green | Yellow/Blue
	const double m_dCR, m_dMG, m_dYB;
};


//------------------
// Stuka Script Input Adjustment Brightness class
class CSIABrightness sealed : public CSIAdjustment {
public:

	// dflt ctor
	CSIABrightness( const double _dBrightnessRatio )
		: m_dBrightnessRatio( _dBrightnessRatio ) {}


private:

	// adjust RGB table
	void Adjust( CRGBAdjustment & _rRGBAdjustment );


private:

	// brightness ratio
	const double m_dBrightnessRatio;
};


//------------------
// Stuka Script Input Adjustment Contrast class
class CSIAContrast sealed : public CSIAdjustment {
public:

	// dflt ctor
	CSIAContrast( const double _dContrastRatio )
		: m_dContrastRatio( _dContrastRatio ) {}


private:

	// adjust RGB table
	void Adjust( CRGBAdjustment & _rRGBAdjustment );


private:

	// contrast ratio
	const double m_dContrastRatio;
};


//------------------
// Stuka Script Input Adjustment Saturation class
class CSIASaturation sealed : public CSIAdjustment {
public:

	// dflt ctor
	CSIASaturation( const double _dSaturationRatio )
		: m_dSaturationRatio( _dSaturationRatio ) {}


private:

	// adjust RGB table
	void Adjust( CRGBAdjustment & _rRGBAdjustment );


private:

	// saturation
	const double m_dSaturationRatio;
};


//------------------
// Stuka Script Input Adjustment Dynamic Noise Reduction class
class CSIADNR sealed : public CSIAdjustment {
public:

	// dflt ctor
	CSIADNR( const DWORD _dwLevel )
		: m_dwLevel( _dwLevel ) {}


private:

	// adjust RGB table
	void Adjust( CRGBAdjustment & _rRGBAdjustment );


private:

	// DNR level (%)
	const DWORD m_dwLevel;
};



// Stuka Script Input Adjustment Vertical Flip class
class CSIAVerticalFlip sealed : public CSIAdjustment {
public:

	// dflt ctor
	CSIAVerticalFlip( const CSIAdjustment::eFlip _eFlip )
		: m_eFlip( _eFlip ) {}


private:

	// adjust RGB table
	void Adjust( CRGBAdjustment & _rRGBAdjustment );


private:

	// flip type
	const CSIAdjustment::eFlip m_eFlip;
};