// A. Sints
#include "stdafx.h"

#include "..\StukaDlg.h"
#include "SInput.h"


// script input adjustment
#include "SIAdjustment.h"


// dtor
CSInput::~CSInput() {

	// remove adjustments
	for( POSITION pos = m_lAdjustment.GetHeadPosition(); pos; )
		delete m_lAdjustment.GetNext( pos );
}



// get focus type from string
const CSInput::eFocus CSInput::GetFocusType( const CString & _sFocus ) {

	if( _sFocus == L"center" )			return IF_CENTER; else
	if( _sFocus == L"left" )			return IF_LEFT; else
	if( _sFocus == L"right" )			return IF_RIGHT; else
	if( _sFocus == L"top" )				return IF_TOP; else
	if( _sFocus == L"bottom" )			return IF_BOTTOM; else
	if( _sFocus == L"top_left" )		return IF_TOP_LEFT; else
	if( _sFocus == L"bottom_left" )		return IF_BOTTOM_LEFT; else
	if( _sFocus == L"top_right" )		return IF_TOP_RIGHT; else
	if( _sFocus == L"bottom_right" )	return IF_BOTTOM_RIGHT; else {
		CStukaDlg::Output( L"Error (SInput): Unknown input focus type '" );
		CStukaDlg::Output( _sFocus );
		CStukaDlg::Output( L"'.\n" );
		return IF_NULL;
	}
}


// parse script
const bool CSInput::Parse( const CString & _sScript ) {

	// Color balance
	int iStart = 0;
	CString sParams;
	while( ExtractCommand( _sScript, L"color_balance", sParams, iStart ) ) {

		int iParamStart = 0;
		CString sdCR, sdMG, sdYB;
		if( !ExtractParam( sParams, sdCR, iParamStart ) ) {
			CStukaDlg::Output( L"Error (SInput): Failed to get color balance CR.\n" );
			return false;
		}
		if( !ExtractParam( sParams, sdMG, iParamStart ) ) {
			CStukaDlg::Output( L"Error (SInput): Failed to get color balance MG.\n" );
			return false;
		}
		if( !ExtractParam( sParams, sdYB, iParamStart ) ) {
			CStukaDlg::Output( L"Error (SInput): Failed to get color balance YB.\n" );
			return false;
		}

		// create adjustment
		m_lAdjustment.AddTail( new CSIAColorBalance( _wtof( sdCR ), _wtof( sdMG ), _wtof( sdYB ) ) );
	}

	// Brightness
	iStart = 0;
	while( ExtractCommand( _sScript, L"brightness", sParams, iStart ) ) {

		int iParamStart = 0;
		CString sdBrightness;
		if( !ExtractParam( sParams, sdBrightness, iParamStart ) ) {
			CStukaDlg::Output( L"Error (SInput): Failed to get brightness.\n" );
			return false;
		}

		// create adjustment
		m_lAdjustment.AddTail( new CSIABrightness( _wtof( sdBrightness ) ) );
	}

	// Contrast
	iStart = 0;
	while( ExtractCommand( _sScript, L"contrast", sParams, iStart ) ) {

		int iParamStart = 0;
		CString sdContrast;
		if( !ExtractParam( sParams, sdContrast, iParamStart ) ) {
			CStukaDlg::Output( L"Error (SInput): Failed to get contrast.\n" );
			return false;
		}

		// create adjustment
		m_lAdjustment.AddTail( new CSIAContrast( _wtof( sdContrast ) ) );
	}

	// Saturation
	iStart = 0;
	while( ExtractCommand( _sScript, L"saturation", sParams, iStart ) ) {

		int iParamStart = 0;
		CString sdSaturation;
		if( !ExtractParam( sParams, sdSaturation, iParamStart ) ) {
			CStukaDlg::Output( L"Error (SInput): Failed to get saturation.\n" );
			return false;
		}

		// create adjustment
		m_lAdjustment.AddTail( new CSIASaturation( _wtof( sdSaturation ) ) );
	}

	// dynamic noise reduction
	iStart = 0;
	while( ExtractCommand( _sScript, L"noise_reduction", sParams, iStart ) ) {

		int iParamStart = 0;
		CString sdLevel;
		if( !ExtractParam( sParams, sdLevel, iParamStart ) ) {
			CStukaDlg::Output( L"Error (SInput): Failed to get DNR.\n" );
			return false;
		}

		// create adjustment
		m_lAdjustment.AddTail( new CSIADNR( _wtoi( sdLevel ) ) );
	}

	// vertical flip
	iStart = 0;
	while( ExtractCommand( _sScript, L"vertical_flip", sParams, iStart ) ) {

		int iParamStart = 0;
		CString seFlip;
		if( !ExtractParam( sParams, seFlip, iParamStart ) ) {
			CStukaDlg::Output( L"Error (SInput): Failed to get vertical flip value.\n" );
			return false;
		}

		// get flip type
		const CSIAdjustment::eFlip eFlip = CSIAdjustment::GetFlipType( seFlip );
		if( eFlip == CSIAdjustment::FT_NULL )
			return false;

		// create adjustment
		m_lAdjustment.AddTail( new CSIAVerticalFlip( eFlip ) );
	}

	// we can now set adjustements
	for( POSITION pos = m_lAdjustment.GetHeadPosition(); pos; )
		const_cast< CSIAdjustment * >( m_lAdjustment.GetNext( pos ) )->Adjust( m_adjustment );

	// ok
	return true;
}


// compute focus point depending of the input dimensions
CPoint CSInput::BuildFocusPoint( const DWORD & _dwWidth, const DWORD & _dwHeight ) {

	// computations
	DWORD dwWidthD2 = _dwWidth >> 1;
	DWORD dwHeightD2 = _dwHeight >> 1;
	DWORD dwWidthD31 = _dwWidth / 3;
	DWORD dwWidthD32 = ( _dwWidth << 1 ) / 3;
	DWORD dwHeightD31 = _dwHeight / 3;
	DWORD dwHeightD32 = ( _dwHeight << 1 ) / 3;

	// focus point depends of focus type
	switch( m_eFocus ) {
		case IF_CENTER:			return CPoint( dwWidthD2, dwHeightD2 ); break;
		case IF_LEFT:			return CPoint( dwWidthD31, dwHeightD2 ); break;
		case IF_RIGHT:			return CPoint( dwWidthD32, dwHeightD2 ); break;
		case IF_TOP:			return CPoint( dwWidthD2, dwHeightD31 ); break;
		case IF_BOTTOM:			return CPoint( dwWidthD2, dwHeightD32 ); break;
		case IF_TOP_LEFT:		return CPoint( dwWidthD31, dwHeightD31 ); break;
		case IF_BOTTOM_LEFT:	return CPoint( dwWidthD31, dwHeightD32 ); break;
		case IF_TOP_RIGHT:		return CPoint( dwWidthD32, dwHeightD31 ); break;
		case IF_BOTTOM_RIGHT:	return CPoint( dwWidthD32, dwHeightD32 ); break;
		default:
			return CPoint( dwWidthD2, dwHeightD2 );
	}
}