// A. Sints
#pragma once

/*
input( "id", "path", focus, anchorMs, TLAnchorMs ) {	// focus: center, left, right, top, bottom, top_left, bottom_left, top_right, bottom_right
	color_balance( CR, MG, YB );
	brightness( brightness );
	contrast( contrast );	
	saturation( saturation );
	noise_reduction( level );
	vertical_flip( flip ); // auto, true, false
}
*/


// extract
#include "Extract.h"

// RGB adjustment
#include "RGBAdjustment.h"


// Stuka Script Input class
class CSInput sealed : public CExtract {
public:

	// input focus type
	enum eFocus{ IF_NULL, IF_CENTER, IF_LEFT, IF_RIGHT, IF_TOP, IF_BOTTOM, IF_TOP_LEFT, IF_BOTTOM_LEFT, IF_TOP_RIGHT, IF_BOTTOM_RIGHT };


public:

	// dflt ctor
	CSInput( LPCTSTR _sID, LPCTSTR _sVideoPath, const eFocus _eFocus, const DWORD _dwAnchorMs, const DWORD _dwTLAnchorMs )
		: m_sID( _sID ), m_sVideoPath( _sVideoPath ), m_eFocus( _eFocus ), m_dwAnchorMs( _dwAnchorMs ), m_dwTLAnchorMs( _dwTLAnchorMs ) {}

	// dtor
	~CSInput();


public:

	// get focus type from string
	static const eFocus GetFocusType( const CString & _sFocus );

	// parse script
	const bool Parse( const CString & _sScript );

	// get ID
	const CString & GetID() const { return m_sID; }

	// get video path
	const CString & GetVideoPath() const { return m_sVideoPath; }

	// get adjustment
	const CRGBAdjustment & GetRGBAdjustment() const { return m_adjustment; }

	// compute focus point depending of the input dimensions
	CPoint BuildFocusPoint( const DWORD & _dwWidth, const DWORD & _dwHeight );

	// get timeline anchor in millisecond
	const int GetAnchorDifference() const { return int( m_dwAnchorMs ) - int( m_dwTLAnchorMs ); }


private:

	// input ID
	const CString m_sID;

	// input video path
	const CString m_sVideoPath;

	// input video focus type
	const eFocus m_eFocus;

	// src anchor in millisecond
	const DWORD m_dwAnchorMs;

	// timeline anchor in millisecond
	const DWORD m_dwTLAnchorMs;
	

private:

	// script input adjustment list (can be empty)
	mutable CList< const CSIAdjustment * > m_lAdjustment;

	// RGB Adjustment
	CRGBAdjustment m_adjustment;
};