// A. Sints
#include "stdafx.h"

#include "..\StukaDlg.h"
#include "SMarker.h"


// script marker display
#include "SMDisplay.h"

// script
#include "Script.h"

// input
#include "SInput.h"


// dtor
CSMarker::~CSMarker() {

	// remove displays
	for( POSITION pos = m_lDisplay.GetHeadPosition(); pos; )
		delete m_lDisplay.GetNext( pos );
}


// parse script
const bool CSMarker::Parse( const CString & _sScript ) {

	// tempo
	int iStart = 0;
	CString sParams;
	if( ExtractCommand( _sScript, L"tempo", sParams, iStart ) ) {

		int iParamStart = 0;
		CString sTempo;
		if( !ExtractParam( sParams, sTempo, iParamStart ) ) {
			CStukaDlg::Output( L"Error (SMarker): Failed to extract tempo.\n" );
			return false;
		}
		m_dwTempo = ( DWORD ) _wtoi( sTempo.GetString() );
	}

	// signature
	iStart = 0;
	if( ExtractCommand( _sScript, L"signature", sParams, iStart ) ) {

		int iParamStart = 0;
		CString sdwBeatCount, sdwBeatUnit;
		if( !ExtractParam( sParams, sdwBeatCount, iParamStart ) ) {
			CStukaDlg::Output( L"Error (SMarker): Failed to extract beat count.\n" );
			return false;
		}
		if( !ExtractParam( sParams, sdwBeatUnit, iParamStart ) ) {
			CStukaDlg::Output( L"Error (SMarker): Failed to extract beat unit.\n" );
			return false;
		}
		m_signature.SetSignature( _wtoi( sdwBeatCount.GetString() ), _wtoi( sdwBeatUnit.GetString() ) );
	}

	// effect
	iStart = 0;
	if( ExtractCommand( _sScript, L"effect", sParams, iStart ) ) {

		int iParamStart = 0;
		CString seEffect;
		if( !ExtractParam( sParams, seEffect, iParamStart ) ) {
			CStukaDlg::Output( L"Error (SMarker): Failed to extract effect.\n" );
			return false;
		}
		// get effect type
		const CSMEffect::eEffect eEffect = CSMEffect::GetEffectType( seEffect );
		if( eEffect == CSMEffect::ME_NULL )
			return false;

		// set effect
		m_effect.SetEffect( eEffect );

		// strobe effect, get the cycle
		if( eEffect == CSMEffect::ME_STROBE ) {

			CString seStrobeCycle;
			if( !ExtractParam( sParams, seStrobeCycle, iParamStart ) ) {
				CStukaDlg::Output( L"Error (SMarker): Failed to extract strobe effect cycle.\n" );
				return false;
			}
			// get cycle type
			const CSMDisplay::eCycle eCycle = CSMDisplay::GetCycleType( seStrobeCycle );
			if( eCycle == CSMDisplay::DC_NULL )
				return false;

			// set strobe cycle
			m_effect.SetStrobeCycle( eCycle );
		}
	}

	// separator
	iStart = 0;
	if( ExtractCommand( _sScript, L"separator", sParams, iStart ) ) {

		int iParamStart = 0;
		CString siSeparatorSize;
		if( !ExtractParam( sParams, siSeparatorSize, iParamStart ) ) {
			CStukaDlg::Output( L"Error (SMarker): Failed to extract separator size.\n" );
			return false;
		}
		m_iSeparatorSize = ( DWORD ) _wtoi( siSeparatorSize.GetString() );
	}

	// display
	iStart = 0;
	while( ExtractCommand( _sScript, L"display", sParams, iStart ) ) {

		int iParamStart = 0;
		CString sRefID;
		CString seCycle( L"none" );
		if( !ExtractParam( sParams, sRefID, iParamStart ) ) {
			CStukaDlg::Output( L"Error (SMarker): Failed to extract display refid.\n" );
			return false;
		}
		ExtractParam( sParams, seCycle, iParamStart, true );

		// get cycle type
		const CSMDisplay::eCycle eCycle = CSMDisplay::GetCycleType( seCycle );
		if( eCycle == CSMDisplay::DC_NULL )
			return false;

		// add display
		m_lDisplay.AddTail( new CSMDisplay( sRefID, eCycle ) );
	}

	// resize
	iStart = 0;
	while( ExtractCommand( _sScript, L"resize", sParams, iStart ) ) {

		int iParamStart = 0;
		CString sRefID;
		CString sdwSize;
		if( !ExtractParam( sParams, sRefID, iParamStart ) ) {
			CStukaDlg::Output( L"Error (SMarker): Failed to extract resize refid.\n" );
			return false;
		}
		if( !ExtractParam( sParams, sdwSize, iParamStart ) ) {
			CStukaDlg::Output( L"Error (SMarker): Failed to extract resize size.\n" );
			return false;
		}

		// add display
		m_mResize.SetAt( sRefID, _wtoi( sdwSize.GetString() ) );
	}

	// speed
	iStart = 0;
	while( ExtractCommand( _sScript, L"speed", sParams, iStart ) ) {

		int iParamStart = 0;
		CString sRefID;
		CString sdwSpeed;
		if( !ExtractParam( sParams, sRefID, iParamStart ) ) {
			CStukaDlg::Output( L"Error (SMarker): Failed to extract speed refid.\n" );
			return false;
		}
		if( !ExtractParam( sParams, sdwSpeed, iParamStart ) ) {
			CStukaDlg::Output( L"Error (SMarker): Failed to extract speed value.\n" );
			return false;
		}

		// add display
		m_mSpeed.SetAt( sRefID, _wtof( sdwSpeed.GetString() ) );
	}

	// ok
	return true;
}


// validate script
const bool CSMarker::Validate() {

	// marker tempo can't be 0 if this is the marker #1
	if( m_dwPosition == 1 && m_dwTempo == 0 ) {
		CStukaDlg::Output( L"Error (SMarker): Failed to validate script: marker tempo can't be 0 if this is the marker #1.\n" );
		return false;
	}

	// time signature can't be 0 if we're on the marker #1
	if( m_dwPosition == 1 && ( m_signature.GetBeatCount() == 0 || m_signature.GetBeatUnit() == 0 ) ) {
		CStukaDlg::Output( L"Error (SMarker): Failed to validate script: time signature can't be 0 if we're on the marker #1.\n" );
		return false;
	}

	// display list can be empty if it's the last element else we need to have at least one element
	if( !m_bTimelineEnd && m_lDisplay.IsEmpty() ) {
		CStukaDlg::Output( L"Error (SMarker): Failed to validate script: display list can only be empty if it's the last element.\n" );
		return false;
	}
	if( m_bTimelineEnd && ( !m_lDisplay.IsEmpty() || m_dwTempo != 0 || m_signature.GetBeatCount() != 0 || m_signature.GetBeatUnit() != 0 ) ) {
		CStukaDlg::Output( L"Error (SMarker): Failed to validate script: the content of the last marker element must be empty.\n" );
		return false;
	}

	// layout can't be none if we're not on the last marker
	if( !m_bTimelineEnd && m_eLayoutType == CLayout::LT_NONE ) {
		CStukaDlg::Output( L"Error (SMarker): Failed to validate script: layout was 'none' on a non terminal marker.\n" );
		return false;
	}
	// layout must be none if we're on the last marker
	if( m_bTimelineEnd && m_eLayoutType != CLayout::LT_NONE ) {
		CStukaDlg::Output( L"Error (SMarker): Failed to validate script: layout must be 'none' on a the terminal marker.\n" );
		return false;
	}

	// ok
	return true;
}


// build resize map
const bool CSMarker::BuildResizeMap() {

	CString sRefID;
	DWORD dwResize;
	for( POSITION pos = m_mResize.GetStartPosition(); pos; ) {
		m_mResize.GetNextAssoc( pos, sRefID, dwResize );
		CList< CSInput * > lInputs;
		if( !const_cast< CScript & >( m_rScript ).BuildInputList( sRefID, lInputs ) ) {
			CStukaDlg::Output( L"Error (SMarker): Failed to build input list.\n" );
			return false;
		}
		// set resize by input ID
		for( POSITION posI = lInputs.GetHeadPosition(); posI; )
			m_mResizeID.SetAt( lInputs.GetNext( posI )->GetID(), dwResize );
	}

	// ok
	return true;
}


// get resize value for a given ID
const DWORD CSMarker::GetResizeValue( LPCTSTR _sRefID ) const {

	CMap< CString, LPCTSTR, DWORD, DWORD >::CPair * pPair = m_mResizeID.PLookup( _sRefID );
	if( pPair == NULL )
		return 100; // 100 %
	else
		return pPair->value;
}


// build speed map
const bool CSMarker::BuildSpeedMap() {

	CString sRefID;
	double dSpeed;
	for( POSITION pos = m_mSpeed.GetStartPosition(); pos; ) {
		m_mSpeed.GetNextAssoc( pos, sRefID, dSpeed );
		CList< CSInput * > lInputs;
		if( !const_cast< CScript & >( m_rScript ).BuildInputList( sRefID, lInputs ) ) {
			CStukaDlg::Output( L"Error (SMarker): Failed to build input list.\n" );
			return false;
		}
		// set speed by input ID
		for( POSITION posI = lInputs.GetHeadPosition(); posI; )
			m_mSpeedID.SetAt( lInputs.GetNext( posI )->GetID(), dSpeed );
	}

	// ok
	return true;
}


// get speed value for a given ID
const double CSMarker::GetSpeedValue( LPCTSTR _sRefID ) const {

	CMap< CString, LPCTSTR, double, double >::CPair * pPair = m_mSpeedID.PLookup( _sRefID );
	if( pPair == NULL )
		return 1.0; // 1.0
	else
		return pPair->value;
}


// get ref id list
void CSMarker::GetRefIDList( CList< CString > & _rRefIDList ) const {

	_rRefIDList.RemoveAll();

	// adds display
	for( POSITION pos = m_lDisplay.GetHeadPosition(); pos; )
		_rRefIDList.AddTail( m_lDisplay.GetNext( pos )->GetRefID() );

	// adds resize map IDs
	for( POSITION pos = m_mResizeID.GetStartPosition(); pos; ) {
		CString sRefID;
		DWORD dwResize;
		m_mResizeID.GetNextAssoc( pos, sRefID, dwResize );
		_rRefIDList.AddTail( sRefID );
	}

	// adds speed map IDs
	for( POSITION pos = m_mSpeedID.GetStartPosition(); pos; ) {
		CString sRefID;
		double dSpeed;
		m_mSpeedID.GetNextAssoc( pos, sRefID, dSpeed );
		_rRefIDList.AddTail( sRefID );
	}
}