// A. Sints
#pragma once

/*
marker( position, layout ) { // layout: see Layout.h
	tempo( tempo );
	signature( beat_count, beat_unit );
	separator( size );
	display( "input_id or group_id", cycle ); // cycle: none, each_measure, each_beat, every_2_beats, every_3_beats
	resize( "input_id or group_id", size_in_percent );
	speed( "input_id or group_id", speed_rate );
	effect( type ); // effect: none, fade_in, fade_out
}
*/

// layout
#include "Layout.h"

// script marker signature
#include "SMSignature.h"

// script marker effect
#include "SMEffect.h"

// display
#include "SMDisplay.h"

// extract
#include "Extract.h"

// fwd script
class CScript;



// Stuka Script Marker class
class CSMarker sealed : public CExtract {
public:

	// dflt ctor
	CSMarker( const CScript & _rScript, const DWORD _dwPosition, const CLayout::eType _eType )
		: m_rScript( _rScript ), m_dwPosition( _dwPosition ), m_eLayoutType( _eType ),
		m_dwTempo( 0 ), m_bTimelineEnd( false ), m_iSeparatorSize( -1 ) {}

	// dtor
	~CSMarker();


public:

	// parse script
	const bool Parse( const CString & _sScript );

	// validate script
	const bool Validate();

	// get position
	const DWORD & GetPosition() const { return m_dwPosition; }

	// set the marker as timeline ending
	void SetTimelineEnd() { m_bTimelineEnd = true; }

	// is timeline end
	const bool IsTimelineEnd() const { return m_bTimelineEnd; }

	// get tempo
	const DWORD & GetTempo() const { return m_dwTempo; }

	// get signature
	const CSMSignature & GetSignature() const { return m_signature; }

	// get display list
	const CList< const CSMDisplay * > & GetDisplayList() const { return m_lDisplay; }

	// get layout type
	const CLayout::eType GetLayoutType() const { return m_eLayoutType; }

	// get effect
	const CSMEffect & GetEffect() const { return m_effect; }

	// get separator size
	const int GetSeparatorSize() const { return m_iSeparatorSize; }

	// build resize map
	const bool BuildResizeMap();

	// get resize value for a given ID
	const DWORD GetResizeValue( LPCTSTR _sRefID ) const;

	// build speed map
	const bool BuildSpeedMap();

	// get speed value for a given ID
	const double GetSpeedValue( LPCTSTR _sRefID ) const;

	// get ref id list
	void GetRefIDList( CList< CString > & _rRefIDList ) const;


private:

	// script reference
	const CScript & m_rScript;

	// layout type (can't be none except on last marker)
	const CLayout::eType m_eLayoutType;

	// marker position
	const DWORD m_dwPosition;

	// this marker is the end of the timeline
	mutable bool m_bTimelineEnd;


private:

	// marker tempo (0 if inherited from previous marker, can't be 0 if this is the marker #1)
	mutable DWORD m_dwTempo;

	// time signature (0: inherit from previous marker, error if we're on the marker #1)
	mutable CSMSignature m_signature;

	// effect (can be none)
	mutable CSMEffect m_effect;

	// separator size (-1: inherit, 0: none, >0 size of separator)
	mutable int m_iSeparatorSize;

	// display list (can be empty if it's the last element else we need to have at least one element)
	mutable CList< const CSMDisplay * > m_lDisplay;

	// resize list (can be empty)
	mutable CMap< CString, LPCTSTR, DWORD, DWORD > m_mResize;
	mutable CMap< CString, LPCTSTR, DWORD, DWORD > m_mResizeID;

	// speed list (can be empty)
	mutable CMap< CString, LPCTSTR, double, double > m_mSpeed;
	mutable CMap< CString, LPCTSTR, double, double > m_mSpeedID;
};