// A. Sints
#pragma once

/*
output( "path", width, height, "audio_path" );

input( "id", "path", focus, anchorMs, TLAnchorMs ) {	// focus: center, left, right, top, bottom, top_left, bottom_left, top_right, bottom_right
	color_balance( CR, MG, YB );
	brightness( brightness );
	contrast( contrast );	
	saturation( saturation );
	noise_reduction( level );
	vertical_flip( flip ); // auto, true, false
}

group( "id" ) {
	"input_id or group_id",
	...
}

debug();

range( begin, end );

marker( position, layout ) { // layout: see Layout.h
	tempo( tempo );
	signature( beat_count, beat_unit );
	separator( size );
	display( "input_id or group_id", cycle ); // cycle: none, each_measure, each_beat, every_2_beats, every_3_beats
	resize( "input_id or group_id", size_in_percent );
	speed( "input_id or group_id", speed_rate );
	effect( type ); // effect: none, fade_in, fade_out
}

overlay( "input_id", loop_count ) {
	capture( left, top, right, bottom );
}

*/


// extract
#include "Extract.h"

// list sort
#include "ListSort.h"


// script input/group/marker
class CSInput;
class CSGroup;
class CSMarker;

// overlay
#include "SOverlay.h"


// Stuka Script class
class CScript sealed : public CExtract {
public:

	// dflt ctor
	CScript( LPCTSTR _sScriptPath )
		: m_sScriptPath( _sScriptPath ), m_dwRangeBegin( 0 ), m_dwRangeEnd( 0 ), m_bDebugMode( false ) {}

	// dtor
	~CScript();


public:

	// parse script
	const bool Parse();

	// validate script
	const bool Validate();

	// get marker list
	const CListSort< const CSMarker * > & GetMarkerList() const { return m_lMarker; }

	// build input list from an input or group id
	const bool BuildInputList( const CString & _sRefID, CList< CSInput * > &_rlInputs );

	// get overlay
	const CSOverlay & GetOverlay() const { return m_overlay; }

	// get output path
	const CString & GetOutputPath() const { return m_sOutputPath; }

	// get output width
	const DWORD & GetOutputWidth() const { return m_dwWidth; }
	// get output height
	const DWORD & GetOutputHeight() const { return m_dwHeight; }

	// get audio path
	const CString & GetAudioPath() const { return m_sAudioPath; }

	// get ranges
	const DWORD & GetRangeBegin() const { return m_dwRangeBegin; }
	const DWORD & GetRangeEnd() const { return m_dwRangeEnd; }

	// is debug
	const bool IsDebug() const { return m_bDebugMode; }


private:

	// remove all comments
	const bool _RemoveComments( CString & _sScript );

	// parse output
	const bool _ParseOutput( const CString & _sScript );

	// parse inputs
	const bool _ParseInputs( const CString & _sScript );

	// parse groups
	const bool _ParseGroups( const CString & _sScript );

	// parse markers
	const bool _ParseMarkers( const CString & _sScript );

	// parse overlay
	const bool _ParseOverlay( const CString & _sScript );

	// parse range
	const bool _ParseRange( const CString & _sScript );

	// parse debug
	const bool _ParseDebug( const CString & _sScript );

	// get interpolation type from string
	int _GetInterpolationType( const CString & _sInterpolationType );


private:

	// build script path
	const CString m_sScriptPath;


private:

	// output avi path
	CString m_sOutputPath;

	// output avi dimensions
	DWORD m_dwWidth;
	DWORD m_dwHeight;

	// audio path (optional)
	CString m_sAudioPath;

	// range
	DWORD m_dwRangeBegin;
	DWORD m_dwRangeEnd;

	// debug mode
	bool m_bDebugMode;

	// script input list (can't be empty)
	mutable CMap< CString, LPCTSTR, const CSInput *, const CSInput * > m_lInput;

	// script group list (can be empty)
	mutable CMap< CString, LPCTSTR, const CSGroup *, const CSGroup * > m_lGroup;

	// script marker list (must have at least two elements: start( 1 ) & end)
	mutable CListSort< const CSMarker * > m_lMarker;

	// overlay (can be empty)
	mutable CSOverlay m_overlay;
};