// A. Sints
#include "stdafx.h"

#include "Thread.h"


// ctor
CThread::CThread() : m_bStop( true ) {

	// no auto deletion
	m_bAutoDelete = FALSE;

	// not stopped
	m_evtStopped.SetEvent();
}


// start
#ifdef _DEBUG
const bool CThread::Start( LPCTSTR _sName, const int _nPriority ) {
#else
const bool CThread::Start( LPCTSTR, const int _nPriority ) {
#endif

	// stop is active
	m_bStop = false;
	m_evtStopped.ResetEvent();

	// create the thread
	if( CreateThread( CREATE_SUSPENDED ) == FALSE )
		return false;

	// set thread priority
	SetThreadPriority( _nPriority );

	#ifdef _DEBUG

		// set thread name (max. 32 chars)
		char szTempName[ 32 ];
		ZeroMemory( szTempName, sizeof( szTempName ) );
		lstrcpynA( szTempName, CStringA( _sName ), 32 );

		// thread name struct
		typedef struct {
			DWORD dwType;		// must be 0x1000
			LPCSTR szName;		// pointer to name in same address space thread name should be limited to 9 characters
			DWORD dwThreadId;	// thread Id, -1 = caller thread
			DWORD dwFlags;		// reserved for future use, must be zero
		} SETTHREADNAMEINFO;

		// fill struct
		SETTHREADNAMEINFO STNInfo;
		ZeroMemory( &STNInfo, sizeof( SETTHREADNAMEINFO ) );
		STNInfo.dwType		= 0x1000;
		STNInfo.szName		= szTempName;
		STNInfo.dwThreadId	= m_nThreadID;
		STNInfo.dwFlags		= 0;

		// raise exception to communicate the new thread name
		RaiseException( 0x406d1388, 0, sizeof( STNInfo ) / sizeof( DWORD ), ( DWORD * ) &STNInfo );

		CString sOut;
		sOut.Format( L"The thread '%s' (0x%x) has started.\n", _sName, m_nThreadID );
		OutputDebugString( sOut );
	#endif

	// resume thread
	return ( ResumeThread() == 1 );
}


// stop
void CThread::Stop() {

	// stop if not already stopped
	if( m_bStop )
		return;
	m_bStop = true;
	// wait for thread
	WaitForSingleObject( m_evtStopped, INFINITE );
	Sleep( 1 );
}


// run
int CThread::Run() {

	// callback until stopped
	while( !m_bStop ) {
		if( !Callback() )
			break;
		Sleep( 1 );
	}
	// stopped
	m_evtStopped.SetEvent();
	return 0;
}