// A. Sints
#include "stdafx.h"

#include "Stuka.h"
#include "StukaDlg.h"

#ifdef _DEBUG
	#define new DEBUG_NEW
#endif


// project
#include "Project.h"

// adjustments
#include "AdjustmentsDlg.h"


// current version
const LPCTSTR CStukaDlg::m_sVersion = L"1.0.9";


// static
HWND CStukaDlg::m_hWndStatic;


// ctor
CStukaDlg::CStukaDlg( CWnd * _pParent )
	: CDialog( CStukaDlg::IDD, _pParent ), m_pStukaProject( NULL ), m_rctClient( 0, 0, 0, 0 ),
	m_pOutputDlg( NULL ), m_bAdjust( false ) {

	// load main frame icon
	m_hIcon = AfxGetApp()->LoadIcon( IDR_MAINFRAME );
}


// on cancel
void CStukaDlg::OnCancel() {

	// delete runner if active
	m_bAdjust = true;
	delete m_pStukaProject;
	if( m_pOutputDlg != NULL ) {
		delete m_pOutputDlg;
		m_pOutputDlg = NULL;
	}
	CDialog::OnCancel();
}


// message map
BEGIN_MESSAGE_MAP( CStukaDlg, CDialog )
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_SIZE()
	ON_WM_MOVE()
	ON_WM_CLOSE()
	ON_MESSAGE( WM_POSTPROGRESS, OnPostProgress )
	ON_MESSAGE( WM_POSTOUTPUT, OnPostOutput )
	ON_MESSAGE( WM_POSTREMAINING, OnPostRemaining )
	ON_MESSAGE( WM_POSTENDBUILD, OnPostEndBuild )
	//}}AFX_MSG_MAP
	ON_BN_CLICKED( IDC_BUILD, &CStukaDlg::OnBnClickedBuild )
	ON_BN_CLICKED( IDC_ADJUST, &CStukaDlg::OnBnClickedAdjust )
END_MESSAGE_MAP()


// DDX/DDV support
void CStukaDlg::DoDataExchange( CDataExchange * _pDX ) {

	CDialog::DoDataExchange( _pDX );
	DDX_Control( _pDX, IDC_OUTPUT, m_out );
	DDX_Control( _pDX, IDC_PROGRESS, m_progress );
	DDX_Control( _pDX, IDC_BUILD, m_btnBuild );
	DDX_Control( _pDX, IDC_ADJUST, m_btnAdjust );
	DDX_Control( _pDX, IDC_PROGRESS_TOTAL, m_ProgressTotal );
	DDX_Control( _pDX, IDC_TXTREMAINING, m_txtRemaining );
	DDX_Control( _pDX, IDC_REMAINING, m_remaining );
}


// on init dlg
BOOL CStukaDlg::OnInitDialog() {

	// call parent
	CDialog::OnInitDialog();

	// HWND
	m_hWndStatic = m_hWnd;

	// set window text
	SetWindowText( L"Stuka - music video creator for studio recording session" );

	// set big/small icon
	SetIcon( m_hIcon, TRUE );
	SetIcon( m_hIcon, FALSE );

	// min rect
	GetWindowRect( m_rctWin );
	GetClientRect( m_rctClient );
	
	// ctrl auto resize
	m_ctrlAutoPool.AddObject( &m_out,			CCtrlAuto::EA_FIXED, CCtrlAuto::EA_FIXED, CCtrlAuto::EA_FIXED,	CCtrlAuto::EA_FIXED );
	m_ctrlAutoPool.AddObject( &m_progress,		CCtrlAuto::EA_FIXED, CCtrlAuto::EA_FIXED, CCtrlAuto::EA_FIXED,	CCtrlAuto::EA_FIXED );
	m_ctrlAutoPool.AddObject( &m_ProgressTotal,	CCtrlAuto::EA_FIXED, CCtrlAuto::EA_FIXED, CCtrlAuto::EA_FIXED,	CCtrlAuto::EA_FIXED );
	m_ctrlAutoPool.AddObject( &m_txtRemaining,	CCtrlAuto::EA_FIXED, CCtrlAuto::EA_STATIC, CCtrlAuto::EA_FIXED,	CCtrlAuto::EA_STATIC );
	m_ctrlAutoPool.AddObject( &m_remaining,		CCtrlAuto::EA_FIXED, CCtrlAuto::EA_STATIC, CCtrlAuto::EA_FIXED,	CCtrlAuto::EA_STATIC );
	m_ctrlAutoPool.AddObject( &m_btnBuild,		CCtrlAuto::EA_FIXED, CCtrlAuto::EA_FLOAT, CCtrlAuto::EA_FIXED,	CCtrlAuto::EA_FIXED );
	m_ctrlAutoPool.AddObject( &m_btnAdjust,		CCtrlAuto::EA_FIXED, CCtrlAuto::EA_FIXED, CCtrlAuto::EA_FIXED,	CCtrlAuto::EA_STATIC );

	// build pool
	m_ctrlAutoPool.Build( m_hWnd );

	// hello world
	CString sHello;
	sHello.Format( L"Welcome in Stuka %s by a.sints\n\n", m_sVersion );
	Output( sHello );

	// focus
	return TRUE;
}


// post progress
LRESULT CStukaDlg::OnPostProgress( WPARAM _u8Progress, LPARAM _eProgressType ) {

	if( m_bAdjust )
		return S_OK;

	// depends on type
	if( _eProgressType == PT_LOCAL ) {
		m_progress.SetPos( _u8Progress );
		return S_OK;
	}
	
	// show output dlg
	if( m_pOutputDlg != NULL )
		m_pOutputDlg->ShowWindow( SW_SHOW );

	m_ProgressTotal.SetPos( _u8Progress );
	if( _u8Progress == 100 )			
		SetWindowText( L"Stuka - music video creator for studio recording session" );
	else {
		CString sWinText;
		sWinText.Format( L"Stuka - %i%%", _u8Progress );
		SetWindowText( sWinText.GetString() );
	}
	return S_OK;
}


// post output
LRESULT CStukaDlg::OnPostOutput( WPARAM _sText, LPARAM ) {

	if( m_bAdjust )
		return S_OK;

	CString sText( ( LPCTSTR ) _sText );
	free( ( void * ) _sText );
	sText.Replace( L"\n", L"\r\n" );
	m_out.SetSel( 0xFFFF, 0xFFFF );
	m_out.ReplaceSel( sText );
	return S_OK;
}


// post remaining
LRESULT CStukaDlg::OnPostRemaining( WPARAM _sRemaining, LPARAM ) {

	if( m_bAdjust )
		return S_OK;
	
	CString sRemaining( ( LPCTSTR ) _sRemaining );
	free( ( void * ) _sRemaining );
	m_remaining.SetWindowText( sRemaining );
	return S_OK;
}


// post end build
LRESULT CStukaDlg::OnPostEndBuild( WPARAM, LPARAM ) {

	if( m_bAdjust )
		return S_OK;

	m_btnBuild.EnableWindow( TRUE );
	if( m_pOutputDlg != NULL )
		m_pOutputDlg->ShowWindow( SW_HIDE );

	// delete project
	delete m_pStukaProject;
	m_pStukaProject = NULL;

	return S_OK;
}


// on paint
void CStukaDlg::OnPaint() {

	if( IsIconic() ) {
		CPaintDC dc( this );
		SendMessage( WM_ICONERASEBKGND, reinterpret_cast< WPARAM >( dc.GetSafeHdc() ), 0 );
		int cxIcon = GetSystemMetrics( SM_CXICON );
		int cyIcon = GetSystemMetrics( SM_CYICON );
		CRect rect;
		GetClientRect( &rect );
		dc.DrawIcon( ( rect.Width() - cxIcon + 1 ) >> 1, ( rect.Height() - cyIcon + 1 ) >> 1, m_hIcon );
	}
	else
		CDialog::OnPaint();
}


// on size change
void CStukaDlg::OnSize( UINT _nType, int _cx, int _cy ) {

	// constraint size
	if( _cx < m_rctClient.Width() || _cy < m_rctClient.Height() ) {
		CRect rctCurr;
		GetWindowRect( rctCurr );
		MoveWindow( CRect( rctCurr.TopLeft(), CSize( m_rctWin.Width(), m_rctWin.Height() ) ) );
		GetClientRect( rctCurr );
		m_ctrlAutoPool.Resize( CSize( rctCurr.Width(), rctCurr.Height() ) );
		CDialog::OnSize( _nType, rctCurr.Width(), rctCurr.Height() );
		MoveOutputDlg();
		Invalidate();
		return;
	}

	// ctrl auto resize
	m_ctrlAutoPool.Resize( CSize( _cx, _cy ) );
	CDialog::OnSize( _nType, _cx, _cy );
	MoveOutputDlg();
	Invalidate();
}


// move
void CStukaDlg::OnMove( int _cx, int _cy ) {
	CDialog::OnMove( _cx, _cy );
	MoveOutputDlg();
}


// Build
void CStukaDlg::OnBnClickedBuild() {

	// load dialog
	CString sAllExt( _T("Stuka script Files (*.txt)|*.txt;*.txt|All Files (*.*)|*.*||") );
	CFileDialog dlg( TRUE, NULL, NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, sAllExt.GetString() );
	if( dlg.DoModal() != IDOK )
		return;

	// gray build button
	m_btnBuild.EnableWindow( FALSE );

	// reset output content
	m_out.SetWindowText( L"" );

	// output dlg
	if( m_pOutputDlg == NULL ) {
		m_pOutputDlg = new COutputDlg( this );
		m_pOutputDlg->Create( COutputDlg::IDD, this );
	}
	m_pOutputDlg->Reset();
	MoveOutputDlg();

	// create stuka project
	delete m_pStukaProject;
	m_pStukaProject = new CProject( dlg.GetPathName(), *m_pOutputDlg );
}


// place output dlg
void CStukaDlg::MoveOutputDlg() {

	if( m_pOutputDlg == NULL )
		return;
	CRect rctParent, rctOut;
	GetWindowRect( rctParent );	
	m_pOutputDlg->GetWindowRect( rctOut );	
	m_pOutputDlg->MoveWindow( rctParent.right, rctParent.top, rctOut.Width(), rctOut.Height() );
}


// adjust
void CStukaDlg::OnBnClickedAdjust() {

	// open adjustents
	CAdjustmentsDlg dlgAdj( this );
	m_bAdjust = true;
	dlgAdj.DoModal();
	m_bAdjust = false;
}
