// A. Sints
#pragma once

#include "afxwin.h"
#include "afxcmn.h"

#include "resource.h"

// project
#include "Project.h"

// AutoPool
#include "CtrlAutoPool.h"

// output dlg
#include "OutputDlg.h"



// --- custom messages
#define WM_POSTPROGRESS			WM_APP + 0x00
#define WM_POSTOUTPUT			WM_APP + 0x01
#define WM_POSTREMAINING		WM_APP + 0x02
#define WM_POSTENDBUILD			WM_APP + 0x03



// CStukaDlg dialog
class CStukaDlg : public CDialog {
public:

	// std ctor
	CStukaDlg( CWnd * _pParent = NULL );

	// dtor
	~CStukaDlg() {}

	// Dialog Data
	enum { IDD = IDD_STUKA_DIALOG };


public:

	// progress
	enum eProgressType { PT_LOCAL, PT_TOTAL };
	static void Progress( unsigned char _u8Progress, const eProgressType _eProgressType = PT_LOCAL ) { ::PostMessage( m_hWndStatic, WM_POSTPROGRESS, ( WPARAM ) _u8Progress, ( LPARAM ) _eProgressType ); }

	// output
	static void Output( LPCTSTR _sText ) { ::PostMessage( m_hWndStatic, WM_POSTOUTPUT, ( WPARAM ) _wcsdup( _sText ), NULL ); }

	// remaining time
	static void Remaining( LPCTSTR _sRemaining ) { ::PostMessage( m_hWndStatic, WM_POSTREMAINING, ( WPARAM ) _wcsdup( _sRemaining ), NULL ); }

	// end the build
	static void EndBuild() { ::PostMessage( m_hWndStatic, WM_POSTENDBUILD, NULL, NULL ); }


private:

	// DDX/DDV support
	virtual void DoDataExchange( CDataExchange * _pDX );

	// on init dlg
	virtual BOOL OnInitDialog();

	// on paint
	afx_msg void OnPaint();

	// on query drag icon
	afx_msg HCURSOR OnQueryDragIcon() { return static_cast<HCURSOR>( m_hIcon ); }

	// build
	afx_msg void OnBnClickedBuild();

	// adjust
	afx_msg void OnBnClickedAdjust();

	// size change
	afx_msg void OnSize( UINT, int, int );

	// move
	afx_msg void OnMove( int, int );

	// on cancel
	afx_msg void OnCancel();

	// place output dlg
	void MoveOutputDlg();

	// --- custom messages

	// post progress
	LRESULT OnPostProgress( WPARAM _u8Progress, LPARAM _eProgressType );

	// post output
	LRESULT OnPostOutput( WPARAM _sText, LPARAM );

	// post remaining
	LRESULT OnPostRemaining( WPARAM _sRemaining, LPARAM );

	// post end build
	LRESULT OnPostEndBuild( WPARAM, LPARAM );


private:

	// static HWND
	static HWND m_hWndStatic;

	// progress bar
	CProgressCtrl m_progress;
	CProgressCtrl m_ProgressTotal;

	// app icon
	HICON m_hIcon;

	// Stuka project
	CProject * m_pStukaProject;

	// output dlg
	COutputDlg * m_pOutputDlg;

	// output
	CEdit m_out;

	// remaining time
	CStatic m_txtRemaining;
	CEdit m_remaining;

	// build button
	CButton m_btnBuild;

	// adjust button
	CButton m_btnAdjust;

	// auto resize pool
	CCtrlAutoPool m_ctrlAutoPool;

	// minimum rect
	CRect m_rctWin, m_rctClient;

	// adjust
	bool m_bAdjust;

	// version
	static const LPCTSTR m_sVersion;

	// message map
	DECLARE_MESSAGE_MAP()
};
